library IEEE;
use IEEE.STD_LOGIC_1164.ALL; 
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--VHDL- Programm fr das CPLD, IC9 auf der BCW im Projekt: CLASSICdigital

--"classicdigital.vhd", Stand: 4.6.2003

--Copyright (c)2003 Manfred Fillinger, CLASSICdigital.de
--Dieses Programm ist ein OPEN- SRC- Programm. Es darf fr private Zwecke 
--verwendete und verndert werden.
--Damit im Prozessor SC12 auf dem BCW das OPEN- SRC- Programm "MP3-Player"
--von Thomas Kindler, kreapc.de verwendet werden kann, wurden auch einige
--Funktionsblcke vom Lattice, IC3 des MP3-Players als Grundlage fr 
--das VHDL- Programm bernommen.    

entity CLASSICdigital is
    port (
	 ALE : in STD_LOGIC;				--ALE vom SC12
    	 nPCS1 : in STD_LOGIC;				--Chip Select 100..1FF 
	 SYNC : in STD_LOGIC;				--Sync-Signal vom MAS
	 nEOD : in STD_LOGIC;				--End of DATA vom MAS
	 nRD : in STD_LOGIC;				--Read-Signal vom SC12
	 nWR : in STD_LOGIC;				--Write-Signal vom SC12
	 RC_IN : in STD_LOGIC;				--Empf. IR-Fernbedienung
	 nRTR : in STD_LOGIC;				--ready to read vom MAS
      --PIO6 : in STD_LOGIC;				--SC12, PIO6 fr MAS PIO aktiv    

    	 nPCS : out STD_LOGIC;				--MAS PIO Select 
	 A1 : out STD_LOGIC;				--Adresse A1
	 A2 : out STD_LOGIC;				--Adresse A2
	 A3 : out STD_LOGIC;				--Adresse A3
	 nATA_CS0 : out STD_LOGIC;			--CS0 an IDE
	 nATA_CS1 : out STD_LOGIC;			--CS1 an IDE
	 nIORD : out STD_LOGIC;				--Read-Signal an IDE
	 nIOWR : out STD_LOGIC;				--Write-Signal an IDE
	 nOE : out STD_LOGIC;				--Out-Enable 74LVC4245 (/OE)
	 OEn : out STD_LOGIC;				--Out-Enable 74LVC241 (OE\)
	 DIR : out STD_LOGIC;				--Direction 74LVC4245
	 DMARQ0 : out STD_LOGIC;				--DMA-Anforderung vom MAS
	 DMARQ1 : out STD_LOGIC;				--DMA-Anforderung IR-Fernb.
	 PR : out STD_LOGIC;				--Port Read- Signal zum MAS
	 LED : out STD_LOGIC;				--blaue LED (high=an)

	 AD0_7 : inout STD_LOGIC_VECTOR(7 downto 0);	--Adress-Datenbus vom/zum SC12
	 D8_15 : inout STD_LOGIC_VECTOR(7 downto 0);	--Datenbus vom/zum IDE (high Byte)
	 D0_7 : out STD_LOGIC_VECTOR(7 downto 0)	--Datenbus zum MAS
	 );
	  
	 attribute pin_assign: string;

	 attribute pin_assign of ALE: signal is "10";
	 attribute pin_assign of nPCS1: signal is "6";
	 attribute pin_assign of SYNC: signal is "41";
	 attribute pin_assign of nEOD: signal is "13";
	 attribute pin_assign of nRD: signal is "2";
	 attribute pin_assign of nWR: signal is "3";
	 attribute pin_assign of RC_IN: signal is "53";
	 attribute pin_assign of nRTR: signal is "12";
	 --attribute pin_assign of PIO6: signal is "39";




	 attribute pin_assign of nPCS: signal is "1";
	 attribute pin_assign of A1: signal is "58";
	 attribute pin_assign of A2: signal is "57";
	 attribute pin_assign of A3: signal is "56";
	 attribute pin_assign of nATA_CS0: signal is "75";
	 attribute pin_assign of nATA_CS1: signal is "61";
	 attribute pin_assign of nIORD: signal is "62";
	 attribute pin_assign of nIOWR: signal is "63";
	 attribute pin_assign of nOE: signal is "14";
	 attribute pin_assign of OEn: signal is "15";
	 attribute pin_assign of DIR: signal is "17";
	 attribute pin_assign of DMARQ0: signal is "55";
	 attribute pin_assign of DMARQ1: signal is "54";
	 attribute pin_assign of PR: signal is "7";
	 attribute pin_assign of LED: signal is "40";

	 attribute pin_assign of D0_7: signal is "26 25 24 23 21 20 19 18";
	 attribute pin_assign of AD0_7: signal is "72 71 70 69 68 67 66 65";
	 attribute pin_assign of D8_15: signal is "34 33 79 80 81 82 83 84";
	  

end CLASSICdigital;

architecture VERHALTEN of CLASSICdigital is

signal A0_7,ADIN0_7,ADOUT0_7,DOUT8_15: STD_LOGIC_VECTOR(7 downto 0);
signal nATA_CSHI,nMP3_CS,nATA_CS0_I,F_500kHZ,RC_1,RC_2,DMARQ0_I: STD_LOGIC;
signal ALE_Z: STD_LOGIC_VECTOR(2 downto 0);
signal MAS_Z: STD_LOGIC_VECTOR(6 downto 0);


begin

--LED <= not nPCS1;				--Signalisation der IDE- Zugriffe auf LED blau
--LED <= RC_1 xor RC_2;			--Signalisation der IR, DMARQ1- Zugriffe auf LED blau
--LED <= DMARQ0_I;				--Signalisation der MP3- Lade- Zugriffe auf LED blau
--LED <= not nMP3_CS;			--Signalisation MAS_CS auf LED blau
LED <= SYNC;					--Signalisation eines MPEG Headers auf LED blau


DIR <= '1'; 					--Direction 74LVC4245: B=A
nIORD <= nRD;					--Readsignal
nIOWR <= nWR;					--Writesignal
OEn <= '0';					--Out-Enable LVC241 (OE\)
nPCS <= '0';					--MAS PIO immer aktiv
--nPCS <= PIO6;				--MAS PIO aktiv, wenn PIO6=LOW
nOE <= '0';					--Out-Enable LVC4245 (/OE)


--Bustreiber SC12 zu CPLD
ADOUT0_7 <= AD0_7;

--Bustreiber CPLD zu SC12
AD0_7 <= ADIN0_7 when  (nRD = '0' and nATA_CSHI = '0' and nPCS1 = '0') else "ZZZZZZZZ";

--Adressenlatch
A0_7 <= AD0_7 when ALE = '1' else A0_7;

--Adressdecoder
--		nATA_CS0 : 0x00, 0x11
--		nATA_CSHI: 0x01, 0x10
--		nATA-CS1 : 0x20
--		nMP3-CS  : 0x30
--		A1       : 0x02
--        A2	    : 0x04
--		A3	    : 0x08
A1 <= '1' when A0_7  = "------10" else '0';
A2 <= '1' when A0_7 (2) = '1' else '0';	  
A3 <= '1' when A0_7 (3) = '1' else '0';
nATA_CS0 <= '0' when nPCS1 = '0' and (A0_7 = "--00---0" or A0_7 = "--01---1") else '1';
nATA_CS0_I <= '0' when nPCS1 = '0' and (A0_7 = "--00---0" or A0_7 = "--01---1") else '1';
nATA_CSHI <= '0' when nPCS1 = '0' and (A0_7 = "--00---1" or A0_7 = "--01---0") else '1';
nATA_CS1 <= '0' when nPCS1 = '0' and A0_7 = "--10---0" else '1';
nMP3_CS <= '0' when nPCS1 = '0' and A0_7 = "--11----" else '1';

--Datenlatch Bit 8-15 pos. Taktflanke CPLD intern
process (nWR)
begin
	if (nWR'event and nWR = '1' and nATA_CSHI = '0') then
		DOUT8_15 <= ADOUT0_7;
	end if;
end process; 	

--Bustreiber D8-15 CPLD zu IDE
D8_15 <= DOUT8_15 when nWR ='0' else "ZZZZZZZZ";

--Datenlatch Bit 8-15 pos. Taktflanke IDE zu CPLD
process (nRD)
begin
	if (nRD'event and nRD = '1' and nATA_CS0_I = '0') then
		ADIN0_7 <= D8_15;
	end if;
end process;

--IR-Fernbedienung 
--Frequenzteiler durch 8 (3 Bit Dualzhler, ALE_Z)
process (ALE)
begin
	if (ALE'event and ALE = '1') then
		ALE_Z <= ALE_Z + 1;
	end if;
end process;
F_500kHz <= ALE_Z (2);			--Bit 3 von ALE_Z
--Datenlatch 1, positive IR- Flanke
process (F_500kHz)
begin
	if (F_500kHz'event and F_500kHz = '1') then
		RC_1 <= RC_IN;
	end if;
end process;
--Datenlatch 2, negative IR- Flanke
process (F_500kHz)
begin
	if (F_500kHz'event and F_500kHz = '1') then
		RC_2 <= RC_1;
	end if;
end process;
--DMA Anforderung1 fr IR
DMARQ1 <= RC_1 xor RC_2;
 	
 	
--Ansteuerung MAS 3587F Decoderbetrieb
--Datenbustreiber CPLD zu MAS
process (nWR, nMP3_CS)
begin
	if (nWR'event and nWR = '1' and nMP3_CS = '0') then
		D0_7 <= ADOUT0_7;
	end if;
end process;

--Schreibtiming PR- Signal, Forderung: > 480ns
--7 Bit Dualzhler mit Reseteingang (MAS_Z)
process (nWR, nMP3_CS, ALE)
begin
	if nWR = '0' and nMP3_CS = '0' then
		MAS_Z <= "0000000";
		elsif (ALE'event and ALE = '1' and MAS_Z (5) = '0' and nRTR = '1') then
		MAS_Z <= MAS_Z + 1;
	end if;
end process;

--Signal PR solange Zhlerwert <= 32 ist
PR <= '1' when MAS_Z = "00-----" or nRTR = '0' else '0';

--DMA- Ansteuerung0 fr MAS laden
DMARQ0 <= '1' when MAS_Z = "0100000" and nEOD ='1' and nRTR = '1' else '0';
DMARQ0_I <= '1' when MAS_Z = "0100000" and nEOD ='1' and nRTR = '1' else '0';

 

end VERHALTEN;
