/*
   CPP-File fr MAS CHIP
*/

#include "i2c_api.h"
#include <stdio.h>
#include <dos.h>

#define MAS 0x3E		//Basisadresse des MAS Chips
#define MASW MAS
#define MASR MAS+1

unsigned char recvbuf2[2];
unsigned char sendbuf3[3];
unsigned char sendbuf4[4];
unsigned char recvbuf4[4];
unsigned char sendbuf5[5];
unsigned char sendbuf7[7];
unsigned char sendbuf9[9];
unsigned char sendbuf11[11];

void mas_setVolume(unsigned char vol);

unsigned char vol=0x7F;
unsigned char bass=0x00;
unsigned char treb=0x00;
unsigned char bal=0x00;

void mapi_sleep(int ms)
{
   union REGS inregs;
   union REGS outregs;
   struct SREGS segregs;

   inregs.x.ax = 0x0900;
   inregs.x.bx = ms;
   int86x(0xAC,&inregs,&outregs,&segregs);
}


unsigned char getVol(void) {
 return vol;
}

unsigned getBass(void) {
 return (bass);
}

unsigned getTreb(void) {
 return (treb);
}

unsigned getBal(void) {
 return (bal);
}


void init_i2c_pins(void) {
//    I2C_select_clock_pin(11);
//    I2C_select_data_pin(10);
    I2C_init ();
}


void Init_Direct_Config (void)
{

  unsigned int transmit=0;

  /******************************************************/
  // write control register
  /******************************************************/

  sendbuf3[0]=0x6A;  //subadress (control)
  sendbuf3[1]=0xBC;  //enable DC/DC1; DC/DC2; DSP core; audio codec
  sendbuf3[2]=0x00;
  transmit = I2C_transmit_block(MAS, (char*)&sendbuf3, 3);   //send datablock

  if(transmit)
  {
	  	printf("\nI2C send block error %d",MAS);
  }
  else
  {
  		printf("\nControl:Sending_SA   %x", sendbuf3[0]);
      printf("\t\tSending_HBD   %x", sendbuf3[1]);
      printf("\tSending_LBD   %x\n", sendbuf3[2]);
  }
  I2C_release();   //Freigabe I2C-Bus

  /******************************************************/
  // read control register
  /******************************************************/

  I2C_transmit_char(MAS, 0x6A);  //subadress (control)
  I2C_receive_block(0x3F, (char*)&recvbuf2, 2);	//receive datablock
  I2C_release();                 //Freigabe I2C-Bus

  printf("\t\t\t\tReceive_HBD   %x", recvbuf2[0]);
  printf("\tReceive_LBD   %x\n", recvbuf2[1]);

  /******************************************************/
  // write dccf register
  /******************************************************/

  sendbuf3[0]=0x76;  //subadress (dccf)
  sendbuf3[1]=0x28;  //Spannung 2: 2.5V
  sendbuf3[2]=0x38;  //Spannung 1: 2.7V
  transmit = I2C_transmit_block(0x3E, (char*)&sendbuf3, 3);	//send datablock

  if(transmit)
  {
	  	printf("\nI2C send block error %x",MAS);
  }
  else
  {
  		printf("\nDCCF:\tSending_SA   %x", sendbuf3[0]);
      printf("\t\tSending_HBD   %x", sendbuf3[1]);
      printf("\tSending_LBD   %x\n", sendbuf3[2]);
  }
  I2C_release();    //I2C-Freigabe

  /******************************************************/
  // read dccf register
  /******************************************************/

  I2C_transmit_char(MAS, 0x76);	  					//send char (subadress data read)
  I2C_receive_block(0x3F, (char*)&recvbuf2, 2);	//receive datablock

  printf("\t\t\t\tReceive_HBD   %x", recvbuf2[0]);
  printf("\tReceive_LBD   %x\n", recvbuf2[1]);

  I2C_release();		//Freigabe I2C-Bus
}

/******************************************************/
// write and read DSP Core
/*****************************************************/

void Init_DSP_Core (void)
{

  unsigned int transmit=0;

  /******************************************************/
  // read IC version
  /******************************************************/

  sendbuf3[0]=0x68;		//subadress (data write)
  sendbuf3[1]=0x70;     //read IC version
  sendbuf3[2]=0x00;
  transmit = I2C_transmit_block(0x3E, (char*)&sendbuf3, 3); 	//send datablock
  I2C_release();

  I2C_transmit_char(MAS, 0x69);                    //send char (subadress data read)
  I2C_receive_block(0x3F, (char*)&recvbuf4, 4);		//receive datablock
  I2C_release();

  printf("\nIC-Version:\t%x\t%x\t%x\t%x",recvbuf4[0],recvbuf4[1],recvbuf4[2],recvbuf4[3]);

  /******************************************************/
  // write application selection to 0
  /******************************************************/

  sendbuf11[0]=0x68;		//subadress (data write)
  sendbuf11[1]=0xE0;    //write D0 memory
  sendbuf11[2]=0x00;
  sendbuf11[3]=0x00;
  sendbuf11[4]=0x01;
  sendbuf11[5]=0x07;    //application selection
  sendbuf11[6]=0xF6;
  sendbuf11[7]=0x00;    //0
  sendbuf11[8]=0x00;
  sendbuf11[9]=0x00;
  sendbuf11[10]=0x00;
  transmit = I2C_transmit_block(0x3E, (char*)&sendbuf11, 11); //send datablock
  I2C_release();
  printf("\nAppSel(send):\t%x\t%x\t%x",sendbuf11[8],sendbuf11[9],sendbuf11[10]);
  if(transmit)
  {
	  	printf("\nI2C send block error %x",MAS);
  }

  /******************************************************/
  // read application running
  /******************************************************/

  sendbuf7[1]=0xC0;		//read D0 memory
  sendbuf7[6]=0xF7;
  transmit = I2C_transmit_block(0x3E, (char*)&sendbuf7, 7); //send datablock
  I2C_release();

  I2C_transmit_char(MAS, 0x69);                 //send char (subadress data read)
  I2C_receive_block(0x3F, (char*)&recvbuf4, 4);	//receive datablock
  I2C_release();

  printf("\nAppSelReset:\t%x\t%x\t%x",recvbuf4[1],recvbuf4[2],recvbuf4[3]);

  /******************************************************/
  // write encoder control
  /******************************************************/

  sendbuf11[6]=0xF0;		//Encoder Control
  sendbuf11[7]=0x00;
  sendbuf11[8]=0x0A;
  sendbuf11[9]=0x02;
  sendbuf11[10]=0xC4;
  transmit = I2C_transmit_block(0x3E, (char*)&sendbuf11, 11);	//send datablock
  I2C_release();
  printf("\nEncoderControl(send):\t%x\t%x\t%x",sendbuf11[8],sendbuf11[9],sendbuf11[10]);
  if(transmit)
  {
	  	printf("\nI2C send block error %x",MAS);
  }

  /******************************************************/
  // write volume output control: LL
  /******************************************************/

  sendbuf11[6]=0xFC;		//memory address
  sendbuf11[7]=0x00;
  sendbuf11[8]=0x08;    //mono left
  sendbuf11[9]=0x00;
  sendbuf11[10]=0x00;
  transmit = I2C_transmit_block(0x3E, (char*)&sendbuf11, 11);
  I2C_release();
  printf("\nLL(send):\t%x\t%x\t%x",sendbuf11[8],sendbuf11[9],sendbuf11[10]);

  /******************************************************/
  // write volume output control: LR
  /******************************************************/

  sendbuf11[6]=0xFD;		//memory address
  sendbuf11[7]=0x00;
  sendbuf11[8]=0x08;    //mono left
  sendbuf11[9]=0x00;
  sendbuf11[10]=0x00;
  transmit = I2C_transmit_block(0x3E, (char*)&sendbuf11, 11);
  I2C_release();
  printf("\nLR(send):\t%x\t%x\t%x",sendbuf11[8],sendbuf11[9],sendbuf11[10]);

  /******************************************************/
  // write volume output control: RL
  /******************************************************/

  sendbuf11[6]=0xFE;		//memory address
  sendbuf11[7]=0x00;
  sendbuf11[8]=0x00;    //mono left
  sendbuf11[9]=0x00;
  sendbuf11[10]=0x00;
  transmit = I2C_transmit_block(0x3E, (char*)&sendbuf11, 11);
  I2C_release();
  printf("\nRL(send):\t%x\t%x\t%x",sendbuf11[8],sendbuf11[9],sendbuf11[10]);

  /******************************************************/
  // write volume output control: RR
  /******************************************************/

  sendbuf11[6]=0xFF;		//memory address
  sendbuf11[7]=0x00;
  sendbuf11[8]=0x00;    //mono left
  sendbuf11[9]=0x00;
  sendbuf11[10]=0x00;
  transmit = I2C_transmit_block(0x3E, (char*)&sendbuf11, 11);
  I2C_release();
  printf("\nRR(send):\t%x\t%x\t%x",sendbuf11[8],sendbuf11[9],sendbuf11[10]);

  /******************************************************/
  // write interface status control
  /******************************************************/

  sendbuf9[0]=0x68;		//subadress (data write)
  sendbuf9[1]=0xE4;		//short write memory
  sendbuf9[2]=0x00;
  sendbuf9[3]=0x00;
  sendbuf9[4]=0x01;
  sendbuf9[5]=0x07;
  sendbuf9[6]=0xF2;     //interface status control
  sendbuf9[7]=0x00;
  sendbuf9[8]=0x2D;
  //sendbuf9[8]=0x0D;
  transmit = I2C_transmit_block(0x3E, (char*)&sendbuf9, 9);	  //send datablock
  I2C_release();
  printf("\nInterfaceControl(send):\t%x\t%x",sendbuf9[7],sendbuf9[8]);
  if(transmit)
  {
	  	printf("\nI2C send block error %x",MAS);
  }

  /******************************************************/
  // write main I/O control   SETTING bit(0) of D0:7F1 !!!
  /******************************************************/

  sendbuf9[6]=0xF1; 		//main I/O control
  sendbuf9[7]=0x01;
  sendbuf9[8]=0x25;
  transmit = I2C_transmit_block(0x3E, (char*)&sendbuf9, 9);		//send datablock
  I2C_release();

  /******************************************************/
  // read main I/O control
  /******************************************************/

  sendbuf7[0]=0x68;		//subadress (data write)
  sendbuf7[1]=0xC4;     //short read memory
  sendbuf7[2]=0x00;
  sendbuf7[3]=0x00;
  sendbuf7[4]=0x01;
  sendbuf7[5]=0x07;
  sendbuf7[6]=0xF1;     //main I/O control
  transmit = I2C_transmit_block(0x3E, (char*)&sendbuf7, 7);		//send datablock
  I2C_release();

  I2C_transmit_char(0x3E, 0x69);                   //send char (subadress data read)
  I2C_receive_block(0x3F, (char*)&recvbuf2, 2);		//receive datablock
  I2C_release();

  printf("\nMainI/OControl:\t%x\t%x",recvbuf2[0],recvbuf2[1]);

  /******************************************************/
  // read encoder control
  /******************************************************/

  sendbuf7[6]=0xF0;		//encoder control
  transmit = I2C_transmit_block(0x3E, (char*)&sendbuf7, 7);	//send datablock
  I2C_release();

  I2C_transmit_char(0x3E, 0x69);                   //send char (subadress data read)
  I2C_receive_block(0x3F, (char*)&recvbuf4, 4);		//receive datablock
  I2C_release();

  printf("\nEncoderControl:\t%x\t%x\t%x",recvbuf4[1],recvbuf4[2],recvbuf4[3]);

  /******************************************************/
  // read interface control
  /******************************************************/

  sendbuf7[1]=0xC4;		//short read memory
  sendbuf7[6]=0xF2;     //interface status control
  transmit = I2C_transmit_block(0x3E, (char*)&sendbuf7, 7);		//send datablock
  I2C_release();

  I2C_transmit_char(0x3E, 0x69);                   //send char (subadress data read)
  I2C_receive_block(0x3F, (char*)&recvbuf2, 2);		//receive datablock
  I2C_release();

  printf("\nInterfaceControl:\t%x\t%x",recvbuf2[0],recvbuf2[1]);

  /******************************************************/
  // read oscillator frequency
  /******************************************************/

  sendbuf7[1]=0xC0;		//read memory
  sendbuf7[6]=0xF3;     //oscillator frequency
  transmit = I2C_transmit_block(0x3E, (char*)&sendbuf7, 7);		//send datablock
  I2C_release();

  I2C_transmit_char(0x3E, 0x69);                   //send char (subadress data read)
  I2C_receive_block(0x3F, (char*)&recvbuf4, 4);		//receive datablock
  I2C_release();

  printf("\nOscillator:\t%x\t%x\t%x",recvbuf4[1],recvbuf4[2],recvbuf4[3]);

  /******************************************************/
  // read output clock configuration
  /******************************************************/

  sendbuf7[6]=0xF4;		//output clock configuration
  transmit = I2C_transmit_block(0x3E, (char*)&sendbuf7, 7);		//send datablock
  I2C_release();

  I2C_transmit_char(0x3E, 0x69);                   //send char (subadress data read)
  I2C_receive_block(0x3F, (char*)&recvbuf4, 4);		//receive datablock
  I2C_release();

  printf("\nOutClkConfig:\t%x\t%x\t%x",recvbuf4[1],recvbuf4[2],recvbuf4[3]);

  /******************************************************/
  // write application selection to decoder
  /******************************************************/

  sendbuf11[0]=0x68;    //subadress (data write)
  sendbuf11[1]=0xE0;		//write memory
  sendbuf11[2]=0x00;
  sendbuf11[3]=0x00;
  sendbuf11[4]=0x01;
  sendbuf11[5]=0x07;
  sendbuf11[6]=0xF6;		//application selection
  sendbuf11[7]=0x00;
  sendbuf11[8]=0x00;
  sendbuf11[9]=0x00;
  sendbuf11[10]=0x0c; //decoder
  //sendbuf11[10]=0x40; //encoder
  transmit = I2C_transmit_block(0x3E, (char*)&sendbuf11, 11); 		//send datablock
  I2C_release();
  printf("\nAppSel(send):\t%x\t%x\t%x",sendbuf11[8],sendbuf11[9],sendbuf11[10]);
  if(transmit)
  {
	  	printf("\nI2C send block error %x",MAS);
  }

  /******************************************************/
  // read application selection
  /******************************************************/

  sendbuf7[1]=0xC0;		//read memory
  sendbuf7[6]=0xF7;     //application running
  transmit = I2C_transmit_block(0x3E, (char*)&sendbuf7, 7);		//send datablock
  I2C_release();

  I2C_transmit_char(0x3E, 0x69);                	//send char (subadress data read)
  I2C_receive_block(0x3F, (char*)&recvbuf4, 4);		//receive datablock
  I2C_release();

  printf("\nAppSelDecoder:\t%x\t%x\t%x",recvbuf4[1],recvbuf4[2],recvbuf4[3]);

}

void Init_Audio_Codec (void)
{

  unsigned int transmit=0;

  /******************************************************/
  // write audio codec configuration
  /******************************************************/

  sendbuf5[0]=0x6c;		//subadress (codec write)
  sendbuf5[1]=0x00;     //audio codec configuration
  sendbuf5[2]=0x00;
  sendbuf5[3]=0x22;
  sendbuf5[4]=0x01;
  transmit = I2C_transmit_block(0x3E, (char*)&sendbuf5, 5);		//send datablock
  I2C_release();

  if(transmit)
  {
	  	printf("\nI2C send block error %d",MAS);
  }

  /******************************************************/
  // read audio codec configuration
  /******************************************************/

  sendbuf3[0]=0x6c;		//subadress (codec write)
  sendbuf3[1]=0x00;     //audio codec configuration
  sendbuf3[2]=0x00;
  transmit = I2C_transmit_block(0x3E, (char*)&sendbuf3, 3);		//send datablock

  I2C_release();

  if(transmit)
  {
	  	printf("\nI2C send block error %d",MAS);
  }

  I2C_transmit_char(0x3E, 0x6d);						//send char (subadress codec read)
  I2C_receive_block(0x3F, (char*)&recvbuf2, 2); //receive datablock
  I2C_release();

  printf("\n\AudioCodecConfig:   \t%x\t%x", recvbuf2[0],recvbuf2[1]);

  /******************************************************/
  // write input mode setting
  /******************************************************/

  sendbuf5[0]=0x6c;		//subadress (codec write)
  sendbuf5[1]=0x00;     //input mode setting
  sendbuf5[2]=0x08;
  sendbuf5[3]=0x80;
  sendbuf5[4]=0x00;
  transmit = I2C_transmit_block(0x3E, (char*)&sendbuf5, 5);	//send datablock
  I2C_release();

  if(transmit)
  {
	  	printf("\nI2C send block error %d",MAS);
  }

  /******************************************************/
  // read input mode setting
  /******************************************************/

  sendbuf3[0]=0x6c;		//subadress (codec write)
  sendbuf3[1]=0x00;     //input mode setting
  sendbuf3[2]=0x08;
  transmit = I2C_transmit_block(0x3E, (char*)&sendbuf3, 3);	//send datablock

  I2C_release();

  if(transmit)
  {
	  	printf("\nI2C send block error %d",MAS);
  }

  I2C_transmit_char(0x3E, 0x6d);                //send char (subadress codec read)
  I2C_receive_block(0x3F, (char*)&recvbuf2, 2);	//receive datablock
  I2C_release();

  printf("\n\InputModeSetting:   \t%x\t%x", recvbuf2[0],recvbuf2[1]);

  /******************************************************/
  // write d/a converter source mixer
  /******************************************************/

  sendbuf5[0]=0x6c;		//subadress (codec write)
  sendbuf5[1]=0x00;     //d/a converter source mixer
  sendbuf5[2]=0x07;
  sendbuf5[3]=0x00;
  sendbuf5[4]=0x7F;  	 //+6dB
  transmit = I2C_transmit_block(0x3E, (char*)&sendbuf5, 5);	//send datablock
  I2C_release();

  if(transmit)
  {
	  	printf("\nI2C send block error %d",MAS);
  }

  /******************************************************/
  // read d/a converter source mixer
  /******************************************************/

  sendbuf3[0]=0x6c;		//subadress (codec write)
  sendbuf3[1]=0x00;     //d/a converter source mixer
  sendbuf3[2]=0x07;
  transmit = I2C_transmit_block(0x3E, (char*)&sendbuf3, 3);	// send datablock

  I2C_release();

  if(transmit)
  {
	  	printf("\nI2C send block error %d",MAS);
  }

  I2C_transmit_char(0x3E, 0x6d);                //send char (read codec)
  I2C_receive_block(0x3F, (char*)&recvbuf2, 2);	//receive datablock
  I2C_release();

  printf("\n\D/AConverterSourceMix:   \t%x\t%x", recvbuf2[0],recvbuf2[1]);

  /******************************************************/
  // write d/a converter output mode
  /******************************************************/

  sendbuf5[0]=0x6c;		//subadress (codec write)
  sendbuf5[1]=0x00;     //d/a converter output mode
  sendbuf5[2]=0x0E;
  sendbuf5[3]=0xC0;
  sendbuf5[4]=0x00;
  transmit = I2C_transmit_block(0x3E, (char*)&sendbuf5, 5);	//send datablock
  I2C_release();

  if(transmit)
  {
	  	printf("\nI2C send block error %d",MAS);
  }

  /******************************************************/
  // read d/a converter output modeg
  /******************************************************/

  sendbuf3[0]=0x6c;		///subadress (codec write)
  sendbuf3[1]=0x00;     //d/a converter output mode
  sendbuf3[2]=0x0E;
  transmit = I2C_transmit_block(0x3E, (char*)&sendbuf3, 3);	//send datablock

  I2C_release();

  if(transmit)
  {
	  	printf("\nI2C send block error %d",MAS);
  }

  I2C_transmit_char(0x3E, 0x6d);                //send char (read codec)
  I2C_receive_block(0x3F, (char*)&recvbuf2, 2);	//receive datablock
  I2C_release();

  printf("\n\D/AConverterOutMode:   \t%x\t%x", recvbuf2[0],recvbuf2[1]);

  /******************************************************/
  // write automatic volume correction (avc)
  /******************************************************/

  sendbuf5[0]=0x6c;		//subadress (codec write)
  sendbuf5[1]=0x00;     //automatic volume correction
  sendbuf5[2]=0x12;
  sendbuf5[3]=0x81;
  sendbuf5[4]=0x00;
  transmit = I2C_transmit_block(0x3E, (char*)&sendbuf5, 5);	//send datablock
  I2C_release();

  if(transmit)
  {
	  	printf("\nI2C send block error %d",MAS);
  }

  /******************************************************/
  // write volume
  /******************************************************/

  sendbuf5[0]=0x6c;		//subadress (codec write)
  sendbuf5[1]=0x00;     //volume control
  sendbuf5[2]=0x10;
  sendbuf5[3]=0x7F;
  sendbuf5[4]=0x00;
  transmit = I2C_transmit_block(0x3E, (char*)&sendbuf5, 5);	//send datablock
  I2C_release();

  if(transmit)
  {
	  	printf("\nI2C send block error %d",MAS);
  }

  /******************************************************/
  // read volume
  /******************************************************/

  sendbuf3[0]=0x6c;		//subadress (codec write)
  sendbuf3[1]=0x00;     //volume control
  sendbuf3[2]=0x10;
  transmit = I2C_transmit_block(0x3E, (char*)&sendbuf3, 3);

  I2C_release();

  if(transmit)
  {
	  	printf("\nI2C send block error %d",MAS);
  }

  I2C_transmit_char(0x3E, 0x6d);               	//send char (read codec)
  I2C_receive_block(0x3F, (char*)&recvbuf2, 2); //receive codec
  I2C_release();

  printf("\n\Volume:   \t%x\t%x", recvbuf2[0],recvbuf2[1]);
}


void INIT_I2C(void) {
  unsigned char res; 				//Ergebnisvariable

  printf("Initialisiere I2C-Bus...\n");
 // I2C_init();     //API Funktion zum initialisieren
  I2C_restart();  //API Funktion zum Reset des Busses

  printf("Suche MAS Chip...\n");
  res = I2C_scan(MAS,0x3E);

  if (res != MAS) {
   printf("Keinen Chip an der erwarteten Adresse gefunden!\n");
  }else{
   printf("Chip gefunden an Adresse 0x%02X\n",res);
  }

}

void MAS_DC(void) {
 unsigned int trans;								//transmit Result
 unsigned char cmd0[3] = {0x6A,0x8C,0x00};
 unsigned char cmd1[2] = {0x78,0x38};     //Zu Sendendes Kommando

 //I2C_transmit_char(MAS,0x6A);
 trans = I2C_transmit_block(MAS,(char*)&cmd0,3);
 I2C_release();

 I2C_transmit_char(MAS,0x76);
 trans = I2C_transmit_block(MAS,(char*)&cmd1,2);

 if(trans) {
  		printf("Fehler beim senden....\n");
 }else{
      printf("Erfolgreich...\n");
}

 I2C_release();
}

void MAS_DCr(void) {
	unsigned int trans;
   unsigned char buf[3];

   I2C_transmit_char(MAS,0x6A);
   I2C_receive_block(MAS+1,buf,2);
   I2C_release();

   printf("buf 6A: 0x%2X 0x%2X\n",buf[0],buf[1]);

   I2C_transmit_char(MAS,0x76);
   I2C_receive_block(MAS+1,buf,2);
   I2C_release();

   printf("buf: 0x%2X 0x%2X\n",buf[0],buf[1]);
}

void mas_basics(void) {

 init_i2c_pins();

 INIT_I2C();
// MAS_DC();
// MAS_DCr();

 printf("Init MAS ....\n");
 MAS_DC();
 Init_Direct_Config();
 printf("Ende Direct....\n");
 mapi_sleep(4);
 Init_DSP_Core();
 printf("Ende DSP Init....\n");
 mapi_sleep(4);
 Init_Audio_Codec();
 printf("Ende Codec Init....\n");
 mapi_sleep(4);
 mas_setVolume(0x60);
 printf("Ende Vol Init....\n");

// Init_Audio_Codec();
}

/************************** VOLUME SERVICE ROUTINE ************************/
/**************************************************************************/
//vol kann HEX 00 bis 7F haben...
void mas_setVolume(unsigned char vol) {
 unsigned char cmd[5];
 int transmit;

 cmd[0] = 0x6C;
 cmd[1] = 0x00;
 cmd[2] = 0x10;
 cmd[3] = vol;
 cmd[4] = 0x00;

 transmit = I2C_transmit_block(MASW,(char*)cmd,5);
 I2C_release();
 if (transmit)
 {
   printf("Volume update faild\n");
 }else{
   printf("Volume update sucsess\n");
 }
}

void mas_incVol(void) {
  vol+=2;
  printf("0x%2X",vol);
  if (vol >= 0x7F) vol = 0x7F;
  mas_setVolume(vol);
}

void mas_decVol(void) {
  if (vol != 0x00)
      vol-=2;
  printf("0x%2X",vol);
  mas_setVolume(vol);
}

/************************** BASS SERVICE ROUTINE ************************/
/**************************************************************************/
//vol kann HEX 60 < 0 > A0 haben...
void mas_setBASS(unsigned char vol) {
 unsigned char cmd[5];
 int transmit;

 cmd[0] = 0x6C;
 cmd[1] = 0x00;
 cmd[2] = 0x14;
 cmd[3] = bass;
 cmd[4] = 0x00;

 transmit = I2C_transmit_block(MASW,(char*)cmd,5);
 I2C_release();
 if (transmit)
 {
   printf("Bass update faild\n");
 }else{
   printf("Bass update sucsess\n");
 }
}


void mas_incBass(void) {
  if (bass !=0x60)
     bass+=8;
  printf("0x%2X",bass);
  mas_setBASS(bass);
}

void mas_decBass(void) {
  if (bass !=0xA0)
      bass-=8;
  printf("0x%2X",bass);
  mas_setBASS(bass);
}

/************************** TERB SERVICE ROUTINE ************************/
/**************************************************************************/
//vol kann HEX 60 < 0 > A0 haben...
void mas_setTREB(unsigned char vol) {
 unsigned char cmd[5];
 int transmit;

 cmd[0] = 0x6C;
 cmd[1] = 0x00;
 cmd[2] = 0x15;
 cmd[3] = treb;
 cmd[4] = 0x00;

 transmit = I2C_transmit_block(MASW,(char*)cmd,5);
 I2C_release();
 if (transmit)
 {
   printf("Treb update faild\n");
 }else{
   printf("Treb update sucsess\n");
 }
}


void mas_incTreb(void) {
  if (treb !=0x60)
     treb+=8;
  printf("0x%2X",treb);
  mas_setTREB(treb);
}

void mas_decTreb(void) {
  if (treb !=0xA0)
      treb-=8;
  printf("0x%2X",treb);
  mas_setTREB(treb);
}


/************************** BALANCE SERVICE ROUTINE ************************/
/**************************************************************************/
//vol kann HEX 60 < 0 > A0 haben...
void mas_setBAL(unsigned char vol) {
 unsigned char cmd[5];
 int transmit;

 cmd[0] = 0x6C;
 cmd[1] = 0x00;
 cmd[2] = 0x11;
 cmd[3] = bal;
 cmd[4] = 0x00;

 transmit = I2C_transmit_block(MASW,(char*)cmd,5);
 I2C_release();
 if (transmit)
 {
   printf("Balance update faild\n");
 }else{
   printf("Balance update sucsess\n");
 }
}


void mas_incBal(void) {
  if (bal !=0x7F)
     bal+=1;
  printf("0x%2X",bal);
  mas_setBAL(bal);
}

void mas_decBal(void) {
  if (bal !=0x80)
      bal-=1;
  printf("0x%2X",bal);
  mas_setBAL(bal);
}

/******************************************************/
// Reset MAS3587F
/******************************************************/

void mas_Reset (void)
{
  	unsigned int transmit=0;
   unsigned char sendbuf5[5], 	sendbuf3[2];

	sendbuf5[0]=0x6c;  	//subadress (codec write)
  	sendbuf5[1]=0x00;  	//audio codec configuration
  	sendbuf5[2]=0x00;
  	sendbuf5[3]=0x22;  	//disable d/a converter
  	sendbuf5[4]=0x00;
   transmit = I2C_transmit_block(0x3E, (char*)&sendbuf5, 5);	//send datablock
  	I2C_release();

  	sendbuf5[0]=0x6c;		//subadress (codec write)
  	sendbuf5[1]=0x00;    //volume control
  	sendbuf5[2]=0x10;
  	sendbuf5[3]=0x00;    //mute
  	sendbuf5[4]=0x00;
  	transmit = I2C_transmit_block(0x3E, (char*)&sendbuf5, 5);	//send datablock
  	I2C_release();

	sendbuf3[0]=0x6A;  //subadress (control)
  	sendbuf3[1]=0x33;  //disable DSP core; audio codec
  	sendbuf3[2]=0x00;
  	transmit = I2C_transmit_block(MAS, (char*)&sendbuf3, 3);		//send datablock

   printf("\r\n\nMAS ausschaltbereit !");

}
