/*  ata.cpp
    interface for ATA devices

    todo: -

    21.05.2000 tk, ata transfer code rewritten, should now be more
                   standard-compliant and does better error handling.
                   - temporarily removed assembler implementation for
                     ATA_PioDataOut().
    20.05.2000 tk, implemented new fast word-write routines. this means
                   a _major_ speedup for pio data out transfers. see PLD
                   schematics for further details.
    02.05.2000 tk, assembler implementations for read/write data.
    24.04.2000 tk, initial implementation.

    Copyright (c)2000 by Thomas Kindler, thomas.kindler@gmx.de

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version. Read the
    full License at http://www.gnu.org/copyleft for more details.
*/

// include files -----
//
#include <time.h>
#include <stdio.h>
#include <string.h>
#include <conio.h>
#include "ide/mpplayer.hpp"
#include "ide/miscide.hpp"
//#include "sc12/sc12.hpp"
#include "ide/ata.hpp"

/***********************
 ** Die Include Datei, Definition und RTX_Sleep_Time sollte
    in den neunen Code durch vorhandenes ersetzt werden!
*/
#include <dos.h>
#define  RTOS_INT    0xAD

int RTX_Sleep_Time(unsigned time_ms)
{
  REGPACK regs;
  regs.r_ax = 0;
  regs.r_bx = time_ms;
  intr( RTOS_INT, &regs );
  //return errorcode
  return regs.r_ax;
}
/**************************/



// wait until BSY is negated
//
ERROR  _ATA_WaitForNotBSY()
{
  time_t  timeout = clock();
  while ( (clock()-timeout) < IDE_TIMEOUT )  {
    uint status = inp(ATAREG_rALTSTATUS);
    if (!(status & ATASTATUS_BSY))
      return ERR_OK;
  }
  return ERR_BSYTIMEOUT;
}

// ATA PIO Data In Transfer
//
ERROR  _ATA_PioDataIn( void *data, ulong length )
{
  if (!(inp(ATAREG_rALTSTATUS) & ATASTATUS_DRQ))
    return ERR_NODATA;

  REP_INSW( data, IDE_BASEADDR, length >> 1 );

  // check for uneven transfer lengths
  if (length & 1)
    ((char*)data)[length-1] = inpw(ATAREG_DATA);

  return ERR_OK;
}

// ATA PIO Data Out Transfer
//
ERROR  _ATA_PioDataOut( void *data, uint length )
{
  if (!(inp(ATAREG_rALTSTATUS) & ATASTATUS_DRQ))
    return ERR_NODATA;

  short *p = (short*)data;
  for (int i=0; i < (length>>1); i++)
    outpw( ATAREG_DATAWORDWRITE, SwapShort(*p++) );

  // check for uneven transfer lengths
  if (length & 1)
    outpw( ATAREG_DATA, ((char*)data)[length-1] );

  return ERR_OK;
}

// Execute ATA command
//
ERROR  ATA_ExecCmd( ATA_COMMAND *cmd )
{
  ERROR error;

/*  printf( "ATA_ExecCmd(): cmd=%X, proto=%s, cy=%02X, hd=%02X, "
          "sn=%02X, sc=%02X, fr=%02X,\n",
          cmd->command,
          cmd->protocol == ATACMD::PIO_IN   ? "PI" :
          cmd->protocol == ATACMD::PIO_OUT  ? "PO" :
          cmd->protocol == ATACMD::NON_DATA ? "ND" :
          cmd->protocol == ATACMD::PACKET   ? "P"  : "n/a",
          cmd->cylinder, cmd->deviceHead, cmd->sectorNumber, cmd->sectorCount,
          cmd->features );
*/
  cmd->status = 0;  cmd->error  = 0;

  // HI1: Check_Status, HI2: Device_Select, HI1: Check_Status ----
  error = _ATA_WaitForNotBSY();
  if (error)  return error;

  outp( ATAREG_DEVICEHEAD, cmd->deviceHead );

  error = _ATA_WaitForNotBSY();
  if (error)  return error;

  // HI3: Write_Parameters ----
  outp( ATAREG_CYLINDERLOW , cmd->cylinder & 0xff );
  outp( ATAREG_CYLINDERHIGH, cmd->cylinder >>   8 );
  outp( ATAREG_DEVICEHEAD  , cmd->deviceHead      );
  outp( ATAREG_SECTORNUMBER, cmd->sectorNumber    );
  outp( ATAREG_SECTORCOUNT , cmd->sectorCount     );
  outp( ATAREG_wFEATURES   , cmd->features        );

  // HI4: Write_Command ----
  outp( ATAREG_wCOMMAND, cmd->command );

  // HND1:, HPIOI1:, HPIOO0: Check_Status / HP0: Check_Status_A ----
  error = _ATA_WaitForNotBSY();
  if (error)  return error;

  switch (cmd->protocol)  {
    case ATA_COMMAND::NON_DATA:
      // no data to transfer, everything's done...
      error = ERR_OK;
      break;

    case ATA_COMMAND::PIO_IN:
      // HPIOI2: Transfer_Data (-->host) ----
      error = _ATA_PioDataIn( cmd->data, cmd->length );
      break;

    case ATA_COMMAND::PIO_OUT:
      // HPIOO1: Transfer_Data (-->device) ----
      error = _ATA_PioDataOut( cmd->data, cmd->length );
      break;

    case ATA_COMMAND::PACKET:
      // HP1: Send_Packet ----
      error = _ATA_PioDataOut( cmd->data, cmd->length );
      break;
  }

  // HI0: Host_Idle (or HP2: Check_Status_B) ----
  //   wait for command completion, read
  //   status and error registers.
  //
  error = _ATA_WaitForNotBSY();
  if (error)  return error;

  cmd->status = inp(ATAREG_rSTATUS);
  cmd->error  = inp(ATAREG_rERROR);

//  printf( "  status=%02X, error=%02X.\n", cmd->status, cmd->error );

  return (cmd->status & ATASTATUS_ERR) ? ERR_ATAERROR : ERR_OK;
}


// Identify ATA and ATAPI devices
//
ERROR ATA_IdentifyDevice( ATA_DEVICEINFO *info )
{
  ERROR   error;
  ushort  id_data[256];

  ATA_COMMAND cmd_identify =
    { ATA_IDENTIFYDEVICE, ATA_COMMAND::PIO_IN,     // command, protocol
      0, 0, 0, 0, 0, id_data, sizeof(id_data) };   // cy, dh, sn, sc, fr

  // check for ATA device first
  error = ATA_ExecCmd( &cmd_identify );
  if (error) {
    // check for ATAPI device
    cmd_identify.command = ATA_IDPACKETDEVICE;
    error = ATA_ExecCmd( &cmd_identify );
    if (error)  return error;
  }

  // general configuration word
  memset( info, 0, sizeof(ATA_DEVICEINFO) );
  info->isATAPI     = ((id_data[0] & 0xc000) == 0x8000);
  info->isRemovable = ((id_data[0] & 0x0080) == 0x0080);
  if (info->isATAPI)
    info->atapiDeviceType = ((id_data[0] >> 8) & 0x1f);

  if (!info->isATAPI) {
    // ata devices use CHS translation
    info->logicalCylinders = id_data[1];
    info->logicalHeads     = id_data[3];
    info->logicalSectors   = id_data[6];
  }

  if (id_data[53] & 0x01)  {
    // get current CHS translation (if bit 0 in word 53 is set)
    info->currentCylinders = id_data[54];
    info->currentHeads     = id_data[55];
    info->currentSectors   = id_data[56];
    info->currentCapacity  = (id_data[58]<<16) + id_data[57];
  }

  if (id_data[53] & 0x02)  {
    // get pio timing (if bit 1 in word 53 is set)
    if (id_data[64] & 0x01)  info->maxPIOMode = 3;
    if (id_data[64] & 0x02)  info->maxPIOMode = 4;
    info->minPIOCycleTime      = id_data[67];
    info->minPIOCycleTimeIORDY = id_data[68];
  } else {
    // otherwise assume PIO mode 0 as default timing
    info->maxPIOMode           = 0;
    info->minPIOCycleTime      = 600;
    info->minPIOCycleTimeIORDY = 600;
  }

  // copy serial, firmware & model identification strings
  for (int i=10; i<=19; i++)  id_data[i] = SwapShort(id_data[i]);
  for (int i=23; i<=26; i++)  id_data[i] = SwapShort(id_data[i]);
  for (int i=27; i<=46; i++)  id_data[i] = SwapShort(id_data[i]);
  IsoToCString( info->serialNumber, (char*)&id_data[10], 20, false );
  IsoToCString( info->firmwareRev , (char*)&id_data[23],  8, false );
  IsoToCString( info->modelName   , (char*)&id_data[27], 40, false );

  return ERR_OK;
}


ERROR  ATA_Init()
{
  // disable INTRQ, software reset
  outp( ATAREG_wDEVCONTROL, ATADEVCTRL_nIEN | ATADEVCTRL_SRST );
  RTX_Sleep_Time(2);
  outp( ATAREG_wDEVCONTROL, ATADEVCTRL_nIEN  );
  RTX_Sleep_Time(100);

  printf( "ATA_Init(): scanning for devices...\n" );

  ATA_DEVICEINFO info;
  ATA_IdentifyDevice( &info );

  printf( "  ide0: %-40s %-8s %-20s\n"
          "  %s %s media device, pio mode %d (%d ns).\n",
          info.modelName, info.firmwareRev, info.serialNumber,
          info.isATAPI     ? "ATAPI"     : "ATA",
          info.isRemovable ? "Removable" : "Fixed",
          info.maxPIOMode, info.minPIOCycleTime
        );

  return ERR_OK;
}
