#ifndef ATA_HPP
#define ATA_HPP

// IDE timeout -----
//   If while polling BSY the device remains in a state where it
//   cannot accept a command for more than 5 seconds, the Host
//   shall time out and reset the device. (at least when using
//   ATAPI devices, i could'nt find this in the ATA specs)
//
#define   IDE_TIMEOUT    (CLOCKS_PER_SEC * 5)

// ATA device register set -----
//   this is hardware dependant, and must be
//   matched to the actual register layout.
//
#define   IDE_BASEADDR          0x100
#define   ATAREG_DATA           (IDE_BASEADDR + 0x00)  /* CS0/000 */
#define   ATAREG_rERROR         (IDE_BASEADDR + 0x02)  /* CS0/001 */
#define   ATAREG_wFEATURES      (IDE_BASEADDR + 0x02)  /* CS0/001 */
#define   ATAREG_SECTORCOUNT    (IDE_BASEADDR + 0x04)  /* CS0/010 */
#define   ATAREG_SECTORNUMBER   (IDE_BASEADDR + 0x06)  /* CS0/011 */
#define   ATAREG_CYLINDERLOW    (IDE_BASEADDR + 0x08)  /* CS0/100 */
#define   ATAREG_CYLINDERHIGH   (IDE_BASEADDR + 0x0a)  /* CS0/101 */
#define   ATAREG_DEVICEHEAD     (IDE_BASEADDR + 0x0c)  /* CS0/110 */
#define   ATAREG_rSTATUS        (IDE_BASEADDR + 0x0e)  /* CS0/111 */
#define   ATAREG_wCOMMAND       (IDE_BASEADDR + 0x0e)  /* CS0/111 */
#define   ATAREG_rALTSTATUS     (IDE_BASEADDR + 0x2c)  /* CS1/110 */
#define   ATAREG_wDEVCONTROL    (IDE_BASEADDR + 0x2c)  /* CS1/110 */

// special reversed word write register. See PLD schematics and
// IDE_PioDataOut() routines for further information...
//
#define   ATAREG_DATAWORDWRITE  (IDE_BASEADDR + 0x10)


// Status register bits -----
//
#define   ATASTATUS_ERR                0x01   /* ERROR */
#define   ATASTATUS_DRQ                0x08   /* DATA REQUEST */
#define   ATASTATUS_DRDY               0x40   /* DEVICE READY */
#define   ATASTATUS_BSY                0x80   /* BUSY */


// Device control register bits -----
//
#define   ATADEVCTRL_SRST              0x04   /* SOFTWARE RESET */
#define   ATADEVCTRL_nIEN              0x02   /* DISABLE INTRQ */


// Device/head register bits -----
//
#define   ATADEVHEAD_DEV1              0x10   /* SELECT DEVICE 1 */


// Error register bits -----
//
#define   ATAERR_NM                    0x02   /* NO MEDIA */
#define   ATAERR_ABRT                  0x04   /* COMMAND ABORT */
#define   ATAERR_MCR                   0x08   /* MEDIA CHANGE REQUEST */
#define   ATAERR_IDNF                  0x10   /* INVALID ADDRESS */
#define   ATAERR_MC                    0x20   /* MEDIA CHANGED */
#define   ATAERR_UNC                   0x40   /* DATA UNCORRECTABLE */


// ATA command codes, as defined in
//   Information Technology - (ATA/ATAPI-5)
//   AT Attachment with Packet Interface - 5
//   T13 Working Draft 1321D, Revision 3, 29 February 2000
//
#define   CFA_ERASESECTORS             0xC0   /* CFA ERASE SECTORS */
#define   CFA_REQUESTEXTENDEDERROR     0x03   /* CFA REQUEST EXTENDED ERROR */
#define   CFA_TRANSLATESECTOR          0x87   /* CFA TRANSLATE SECTOR */
#define   CFA_WRITEMULTIPLEWOERASE     0xCD   /* CFA WRITE SECTOR MULPTIPE W/OUT ERASE */
#define   CFA_WRITESECTORSWOERASE      0x38   /* CFA WRITE SECTORS W/OUT ERASE */
#define   ATA_CHECKPOWERMODE           0xE5  	/* CHECK POWER MODE */
#define   ATA_DEVICERESET              0x08  	/* DEVICE RESET */
#define   ATA_DLMICROCODE              0x92  	/* DOWNLOAD MICROCODE */
#define   ATA_EXECDEVDIAG              0x90  	/* EXECUTE DEVICE DIAGNOSTIC */
#define   ATA_FLUSHCACHE               0xE7  	/* FLUSH CACHE */
#define   ATA_GETMEDIASTATUS           0xDA  	/* GET MEDIA STATUS */
#define   ATA_IDENTIFYDEVICE           0xEC  	/* IDENTIFY DEVICE */
#define   ATA_IDLE                     0xE3  	/* IDLE */
#define   ATA_IDLEIMMEDIATE            0xE1  	/* IDLE IMMEDIATE */
#define   ATA_IDPACKETDEVICE           0xA1  	/* IDENTIFY PACKET DEVICE */
#define   ATA_INITDEVPARAM             0x91  	/* INITIALIZE DEVICE PARAMETERS */
#define   ATA_MEDIAEJECT               0xED  	/* MEDIA EJECT */
#define   ATA_MEDIALOCK                0xDE  	/* MEDIA LOCK */
#define   ATA_MEDIAUNLOCK              0xDF  	/* MEDIA UNLOCK */
#define   ATA_NOP                      0x00   /* NOP */
#define   ATA_PACKET                   0xA0  	/* PACKET */
#define   ATA_READBUFFER               0xE4  	/* READ BUFFER */
#define   ATA_READDMA                  0xC8  	/* READ DMA */
#define   ATA_READDMAQUEUED            0xC7  	/* READ DMA QUEUED */
#define   ATA_READMULTIPLE             0xC4  	/* READ MULTIPLE */
#define   ATA_READNATIVEMAXADDR        0xF8  	/* READ NATIVE MAX ADDRESS */
#define   ATA_READSECTOR               0x20  	/* READ SECTOR(S) */
#define   ATA_READVERIFYSECTOR         0x40  	/* READ VERIFY SECTOR(S) */
#define   ATA_SECURITYDISABLEPWD       0xF6   /* SECURITY DISABLE PASSWORD */
#define   ATA_SECURITYERASEPREP        0xF3  	/* SECURITY ERASE PREPARE */
#define   ATA_SECURITYERASEUNIT        0xF4  	/* SECURITY ERASE UNIT */
#define   ATA_SECURITYFREEZELOCK       0xF5  	/* SECURITY FREEZE LOCK */
#define   ATA_SECURITYSETPASSWD        0xF1  	/* SECURITY SET PASSWORD */
#define   ATA_SECURITYUNLOCK           0xF2  	/* SECURITY UNLOCK */
#define   ATA_SEEK                     0x70  	/* SEEK */
#define   ATA_SERVICE                  0xA2  	/* SERVICE */
#define   ATA_SETFEATURES              0xEF  	/* SET FEATURES */
#define   ATA_SETMAXADDRESS            0xF9 	/* SET MAX ADDRESS*/
#define   ATA_SETMULTIPLEMODE          0xC6  	/* SET MULTIPLE MODE */
#define   ATA_SLEEP                    0xE6  	/* SLEEP */
#define   ATA_SMART                    0xB0  	/* SMART */
#define   ATA_STANDBY                  0xE2  	/* STANDBY */
#define   ATA_STANDBYIMMEDIATE         0xE0  	/* STANDBY IMMEDIATE */
#define   ATA_WRITEBUFFER              0xE8  	/* WRITE BUFFER */
#define   ATA_WRITEDMA                 0xCA  	/* WRITE DMA */
#define   ATA_WRITEDMAQUEUED           0xCC  	/* WRITE DMA QUEUED */
#define   ATA_WRITEMULTIPLE            0xC5  	/* WRITE MULTIPLE */
#define   ATA_WRITESECTOR              0x30  	/* WRITE SECTOR(S) */


typedef struct {
  int       command;
  enum    { PIO_IN, PIO_OUT, NON_DATA, PACKET } protocol;

  // command parameters ----
  unsigned int cylinder;
  int      deviceHead;
  int      sectorNumber;
  int      sectorCount;
  int      features;

  void    *data;
  unsigned int  length;

  int      status, error;   // ata return codes
} ATA_COMMAND;


typedef struct  {
  bool    isATAPI;
  bool    isRemovable;

  int     atapiDeviceType;

  unsigned int    logicalCylinders, logicalHeads, logicalSectors;
  unsigned int    currentCylinders, currentHeads, currentSectors;
  unsigned long   currentCapacity;

  int     maxPIOMode;
  int     minPIOCycleTime;
  int     minPIOCycleTimeIORDY;

  char    serialNumber[20+1];
  char    firmwareRev[8+1];
  char    modelName[40+1];
} ATA_DEVICEINFO;

extern  ERROR  _ATA_WaitForNotBSY();
extern  ERROR  _ATA_PioDataIn( void *data, unsigned long length );
extern  ERROR  _ATA_PioDataOut( void *data, unsigned int length );

extern  ERROR  ATA_ExecCmd( ATA_COMMAND *cmd );
extern  ERROR  ATA_IdentifyDevice( ATA_DEVICEINFO *info );
extern  ERROR  ATA_Init();

#endif
