/*  atapi.cpp
    interface for ATAPI devices

    todo: -

    22.05.2000 tk, source forked from ide.cpp.

    Copyright (c)2000 by Thomas Kindler, thomas.kindler@gmx.de

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version. Read the
    full License at http://www.gnu.org/copyleft for more details.
*/

// include files -----
//
#include "ide/mpplayer.hpp"
#include "conio.h";
#include "ide/ata.hpp"
#include "ide/atapi.hpp"
#include "sc12/rtos.h"


// ATAPI PIO Data In Transfer
//
static ERROR  ATAPI_PioDataIn( void *data, unsigned long length )
{
  if (  !(inp(ATAREG_rSTATUS)      & ATASTATUS_DRQ  )
      || (inp(ATAPIREG_rINTREASON) & ATAPIREASON_CoD)
      ||!(inp(ATAPIREG_rINTREASON) & ATAPIREASON_IO ))
    return ERR_NODATA;

  ERROR  error;

  while (length > 0)  {
    //Warten!
    error = _ATA_WaitForNotBSY();
    if (error)  return error;
    //RTX_Sleep_Time(50);
    
    ulong drqlen =  (inp(ATAPIREG_BYTECOUNTLOW ) & 0xff)
                   +(inp(ATAPIREG_BYTECOUNTHIGH) <<   8);

    if (drqlen>length)
      drqlen=length;

    error = _ATA_PioDataIn(data, (uint)drqlen);
    if (error)  break;

    data    = (char*)data+(uint)drqlen;
    length -= drqlen;
  }
  return error;
}


// ATAPI PIO Data Out Transfer
//
static ERROR  ATAPI_PioDataOut( void *data, ulong length )
{
  if (  !(inp(ATAREG_rSTATUS)      & ATASTATUS_DRQ  )
      || (inp(ATAPIREG_rINTREASON) & ATAPIREASON_CoD)
      || (inp(ATAPIREG_rINTREASON) & ATAPIREASON_IO ))
    return ERR_NODATA;

  ERROR  error;
  while  (length > 0)  {
    ulong drqlen =  (inp( ATAPIREG_BYTECOUNTLOW  ) & 0xff)
                   +(inp( ATAPIREG_BYTECOUNTHIGH ) <<   8);

    if (drqlen>length)
      drqlen=length;

    error = _ATA_PioDataOut(data, (uint)drqlen);
    if (error)  break;

    data    = (char*)data+(uint)drqlen;
    length -= drqlen;
  }
  return error;
}


// Request ATAPI Sense information
//
static  ERROR  ATAPI_GetSenseInfo( int *asc, int *ascq )
{
  ERROR  error;
  char   sensedata[18];
  ATAPI_COMMAND  cmd_request_sense =
    { { ATAPI_REQUESTSENSE, 0, 0, 0, 18, 0, 0, 0, 0, 0, 0, 0  },
      ATAPI_COMMAND::PIO_IN, sensedata, sizeof(sensedata)  };

  *asc = 0;  *ascq = 0;
  error = ATAPI_ExecCmd( &cmd_request_sense );
  if (error)  return error;

  *asc  = sensedata[12];
  *ascq = sensedata[13];

  return ERR_OK;
}


// Execute ATAPI Packet command
//
ERROR  ATAPI_ExecCmd( ATAPI_COMMAND *cmd )
{
  ERROR   error;
  ATA_COMMAND   cmd_packet =
    { ATA_PACKET, ATA_COMMAND::PACKET,          // command, protocol
      0xFFFE, 0, 0, 0, 0, cmd->packet, 12 };    // cy, dh, sn, sc, fr

/*  printf( "IDE_ExecPacketCmd():" );
  for (int i=0; i<12; i++)
    printf( " %02x", cmd->packet[i]);
  printf( ", proto=%s\n",
          cmd->protocol == ATACMD::PIO_IN   ? "PI" :
          cmd->protocol == ATACMD::PIO_OUT  ? "PO" :
          cmd->protocol == ATACMD::NON_DATA ? "ND" : "n/a" );
*/
  // Execute packet command, copy return codes ----
  cmd->asc = 0; cmd->ascq = 0;
  error = ATA_ExecCmd( &cmd_packet );

  cmd->status = cmd_packet.status;
  cmd->error  = cmd_packet.error;

  if (!error)  {
    switch (cmd->protocol)  {
      case  ATAPI_COMMAND::NON_DATA:
        // no data to transfer, everything's done...
        error = ERR_OK;
        break;

      case  ATAPI_COMMAND::PIO_IN:
        // HP4: Transfer_Data (--> host) ----
        error = ATAPI_PioDataIn( cmd->data, cmd->length );
        break;

      case  ATAPI_COMMAND::PIO_OUT:
        // HP4: Transfer_Data (--> device) ----
        error = ATAPI_PioDataOut( cmd->data, cmd->length );
        break;
    }
    // HP2: Check_Status_B ----
    //   wait for command completion, read status
    //   and error registers, get sense keys.
    error = _ATA_WaitForNotBSY();
    if (error)  return error;
    cmd->status = inp(ATAREG_rSTATUS);
    cmd->error  = inp(ATAREG_rERROR );
  }

  if (cmd->status & ATASTATUS_ERR)  {
    // avoid recursive request sense deadlock!!
    if (cmd->packet[0] != ATAPI_REQUESTSENSE)
      ATAPI_GetSenseInfo( (int *)&cmd->asc, (int *)&cmd->ascq );
  }

/*  printf( "  status=%02X, error=%02X, asc=%02X, ascq=%02X.\n",
          cmd->status, cmd->error, cmd->asc, cmd->ascq );  */

  return (cmd->status & ATASTATUS_ERR) ? ERR_ATAERROR : ERR_OK;
}

