#ifndef  ATAPI_HPP
#define  ATAPI_HPP

// ATAPI registers -----
//   these are just alias names for  ATA regs
//
#define  ATAPIREG_rINTREASON          ATAREG_SECTORCOUNT
#define  ATAPIREG_BYTECOUNTLOW        ATAREG_CYLINDERLOW
#define  ATAPIREG_BYTECOUNTHIGH       ATAREG_CYLINDERHIGH

// Status register bits -----
//
#define  ATAPISTATUS_CHECK            0x01
#define  ATAPISTATUS_CORR             0x04
#define  ATAPISTATUS_SERVICE          0x10
#define  ATAPISTATUS_DSC              0x10
#define  ATAPISTATUS_DMAREADY         0x20
#define  ATAPISTATUS_DF               0x20
#define  ATAPISTATUS_DRDY             0x40

// Error register bits -----
//
#define  ATAPIERR_ILI                 0x01   /* ILLEGAL LENGTH INDICATOR */
#define  ATAPIERR_EOM                 0x02   /* END OF MEDIA */
#define  ATAPIERR_ABRT                0x04   /* ABORT */
#define  ATAPIERR_MCR                 0x08   /* MEDIA CHANGE REQUEST */
#define  ATAPIERR_SENSEKEY            0xf0

// ATAPI error register sense keys -----
//
#define  ATAPISENSE_NOSENSE              0
#define  ATAPISENSE_RECOVEREDERROR       1
#define  ATAPISENSE_NOTREADY             2
#define  ATAPISENSE_MEDIUMERROR          3
#define  ATAPISENSE_HARDWAREERROR        4
#define  ATAPISENSE_ILLEGALREQUEST       5
#define  ATAPISENSE_UNITATTENTION        6
#define  ATAPISENSE_DATAPROTECT          7
#define  ATAPISENSE_ABORTEDCOMMAND      11
#define  ATAPISENSE_MISCOMPARE          14

// ATAPI interrupt reason register ----
//
#define  ATAPIREASON_CoD              0x01   /* 0=DATA   / 1=CODE  */
#define  ATAPIREASON_IO               0x02   /* 0=OUTPUT / 1=INPUT */
#define  ATAPIREASON_REL              0x04   /* RELEASE */

// ATAPI Packet Commands for CD-ROM Devices
//
#define  ATAPI_INQUIRY                0x12    /* INQUIRY */
#define  ATAPI_LOADUNLOADCD           0xa6    /* LOAD/UNLOAD CD */
#define  ATAPI_MECHANISMSTATUS        0xbd    /* MECHANISM STATUS */
#define  ATAPI_MODESELECT10           0x55    /* MODE SELECT (10) */
#define  ATAPI_MODESENSE10            0x5a    /* MODE SENSE (10) */
#define  ATAPI_PAUSERESUME            0x4b    /* PAUSE/RESUME */
#define  ATAPI_PLAYAUDIO10            0x45    /* PLAY AUDIO (10) */
#define  ATAPI_PLAYAUDIOMSF           0x47    /* PLAY AUDIO MSF */
#define  ATAPI_PLAYCD                 0xbc    /* PLAY CD */
#define  ATAPI_MEDIUMREMOVAL          0x1e    /* ALLOW/PREVENT MEDIUM REMOVAL */
#define  ATAPI_READ10                 0x28    /* READ (10) */
#define  ATAPI_READ12                 0xa8    /* READ (12) */
#define  ATAPI_READCDROMCAPACITY      0x25    /* READ CD-ROM CAPACITY */
#define  ATAPI_READCD                 0xbe    /* READ CD */
#define  ATAPI_READCDMSF              0xb9    /* READ CD MSF */
#define  ATAPI_READHEADER             0x44    /* READ HEADER */
#define  ATAPI_READSUBCHANNEL         0x42    /* READ SUB-CHANNEL */
#define  ATAPI_READTOC                0x43    /* READ TOC */
#define  ATAPI_REQUESTSENSE           0x03    /* REQUEST SENSE */
#define  ATAPI_SCAN                   0xba    /* SCAN */
#define  ATAPI_SEEK                   0x2b    /* SEEK */
#define  ATAPI_SETCDSPEED             0xbb    /* SET CD SPEED */
#define  ATAPI_STOPPLAYSCAN           0x4e    /* STOP PLAY / SCAN */
#define  ATAPI_STARTSTOPUNIT          0x1b    /* START STOP UNIT */
#define  ATAPI_TESTUNITREADY          0x00    /* TEST UNIT READY */


typedef struct {
  unsigned char   packet[12];
  enum    { PIO_IN, PIO_OUT, NON_DATA } protocol;

  void     *data;
  unsigned long  length;

  uint       status, error;   // ata return codes
  uint       asc, ascq;       // atapi sense codes
} ATAPI_COMMAND;


extern ERROR ATAPI_ExecCmd( ATAPI_COMMAND *cmd );
#endif
