/*  cdrom.cpp
    command layer atapi cd-rom interface

    todo:  CD_ListTOC() gives bogo results (sometimes)!

    25.12.2001: sf, new AudioCD Feature Scan/Pause_Resume/
    24.12.2001: sf, some bugfixes in CD_ReadTOC
    23.12.2001: sf, new funktion PlayAudio for LBA-Adressing
    28.05.1999: tk, initial version.

    Copyright (c)2000 by Thomas Kindler, thomas.kindler@gmx.de

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version. Read the
    full License at http://www.gnu.org/copyleft for more details.
*/

// include files ---------------
//
#include <string.h>
#include <stdio.h>
#include "ide/mpplayer.hpp"
#include "ide/miscide.hpp"
#include "ide/atapi.hpp"
#include "ide/cdrom.hpp"
#include "sc12/rtos.h"

typedef struct {
    uchar   reserved;
    uchar   Mode;
    ushort  length;
    uchar   Format;
    uchar   Adress;
    uchar   Tracknr;
    uchar   Indexnr;
    ulong   MSF;
    ulong   LBA;
} CurrPos;

struct  tracki  {
    uchar   reserved;
    uchar   adr_control;
    uchar   track_number;
    uchar   reserved3;
    ulong   lba_address;
  };

typedef struct  {
  ushort  length;
  uchar   first_track;
  uchar   last_track;
  tracki  track[100];
} CD_TOC_FORMAT00;


// lba/msf conversion routines -----
//
CD_MSF  LBA_To_MSF( ulong lba )
{
  CD_MSF  msf;
  lba        += 150;
  msf.minute  = (int)(lba / CD_FRAMES_PER_MINUTE);
  lba         = lba % CD_FRAMES_PER_MINUTE;
  msf.second  = (int)(lba / CD_FRAMES_PER_SECOND);
  lba         = lba % CD_FRAMES_PER_SECOND;
  msf.frame   = (int)lba;
  return  msf;
}

ulong MSF_To_LBA( CD_MSF  msf )
{
  return  msf.minute * CD_FRAMES_PER_MINUTE +
          msf.second * CD_FRAMES_PER_SECOND +
          msf.frame - 150;
}

ERROR CD_ReadTOC( CD_TOC *toc )
{
  ERROR  error;
  CD_TOC_FORMAT00  toc_data;
  // read toc, pp. 169

  //*SF Saubere Arbeitsgrundlage!
  memset( &toc_data, 0x00, sizeof(toc_data) );

  ATAPI_COMMAND  cmd_readtoc =
    { { ATAPI_READTOC, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,
        (char)(sizeof(toc_data) >> 8), sizeof(toc_data) & 0xff,
        0x00, 0x00, 0x00 }, ATAPI_COMMAND::PIO_IN,
        &toc_data, sizeof(toc_data)
    };

  error = ATAPI_ExecCmd( &cmd_readtoc );

  if (error) return error;

  // endianess conversion -----
  toc_data.length = SwapShort(toc_data.length);
  for (int i=0; i<100; i++)
   toc_data.track[i].lba_address = SwapLong( toc_data.track[i].lba_address );


  // fill in our toc-structure -----
  memset( toc, 0x00, sizeof(CD_TOC) );

  toc->num_tracks = toc_data.last_track - toc_data.first_track +1 ;
  //toc->num_tracks = 100;

  for (int i=0; i < toc->num_tracks; i++)  {
    if (toc_data.track[i].adr_control & 4)  {
      toc->track[i].data = true;
      toc->num_data_tracks++;
    } else {
      toc->track[i].data = false;
      toc->num_audio_tracks++;
    }
    toc->track[i].address = toc_data.track[i].lba_address;
    toc->track[i].length  = toc_data.track[i+1].lba_address
                            -toc_data.track[i  ].lba_address;

    toc->volume_size += toc->track[i].length;
  }
  return ERR_OK;
}


ERROR CD_SetSpeed( int  kb_per_sec )
{
  // set cd speed, pp. 193
  ATAPI_COMMAND  cmd_setspeed =
    { { ATAPI_SETCDSPEED,  0x00,
        kb_per_sec >> 8,  kb_per_sec,
        0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00 },
      ATAPI_COMMAND::NON_DATA };

  return  ATAPI_ExecCmd( &cmd_setspeed );
}

ERROR CD_SetMaxSpeed( void ) //Max Speed!
{
  // set cd speed, pp. 193
  ATAPI_COMMAND  cmd_setspeed =
    { { ATAPI_SETCDSPEED,  0x00,
        0xFF, 0xFF,
        0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00 },
      ATAPI_COMMAND::NON_DATA };

  return  ATAPI_ExecCmd( &cmd_setspeed );
}

ERROR CD_PlayAudioMSF( CD_MSF  *start, CD_MSF  *end )
{
  // play audio msf, pp. 128
  ATAPI_COMMAND  cmd_playaudiomsf =
    { { 0x47, 0x00, 0x00,
        start->minute, start->second, start->frame,
        end->minute  , end->second  , end->frame,
        0x00, 0x00, 0x00 }, ATAPI_COMMAND::NON_DATA };
  return  ATAPI_ExecCmd( &cmd_playaudiomsf );
}

ERROR CD_PlayAudioLBA( ulong start, ulong len)
{
  // play audio msf, pp. 128
  // Spielt aber nach LBA Adressen ;-).
  // BugFixed: Wenn 0x45 als erster Parameter sind maximal 0xFFFF Blocks zu spielen
  //
  ATAPI_COMMAND  cmd_playaudiolba =
    { { 0x45, 0x00, (char)(start >> 24), (char)(start >> 16),
        (char)(start >> 8), (char)start,
        (char)(len >> 24), (char)(len >> 16),
        (char)(len >> 8) , (char)(len),
        0x01, 0x00}, ATAPI_COMMAND::NON_DATA };
  return  ATAPI_ExecCmd( &cmd_playaudiolba );
}

ERROR CD_Scan( ulong start, uint dir)
{
  // Scan audio cd, fast forward oder fast-reversed wenn (dir !=0)
  // p. 187
  uchar DIRECT=0;
  if (dir !=0)
     DIRECT=0x10;

  ATAPI_COMMAND  cmd_scancd =
    { { 0xBA, DIRECT, (char)(start >> 24), (char)(start >> 16),
        (char)(start >> 8), (char)start, 0x00,
        0x00, 0x00, 0x00,
        0x00, 0x00}, ATAPI_COMMAND::NON_DATA };
  return  ATAPI_ExecCmd( &cmd_scancd );
}

ERROR CD_Pause(bool pause)
{
  // Pause audio cd, Hold it, or start play from Holdposition (RESUME = 0x01)
  // p. 123
  uchar RESUME=0x00;
  if (pause == false)
      RESUME=0x01;

  ATAPI_COMMAND  cmd_pausecd =
    { { 0x4B, 0x00, 0x00,
        0x00, 0x00, 0x00,
        0x00, 0x00, RESUME,
        0x00, 0x00, 0x00}, ATAPI_COMMAND::NON_DATA };
  return  ATAPI_ExecCmd( &cmd_pausecd );
}

ERROR  CD_Position(ulong &pos)
{

  ERROR  error;
  CurrPos  Mdata;
  // read toc, pp. 169

  //*SF Saubere Arbeitsgrundlage!
  memset( &Mdata, 0x00, sizeof(Mdata) );
  /****************************
   Also wenn QSub = 1 dann kommen Daten, aber noch nicht dass was ich will ;-)
   Die MSF Daten sehen aber recht nett aus ?!
  *****************************/
  ATAPI_COMMAND  cmd_readmech =
    { { 0x42, 0x00, 0x40, 0x01,
        0x00, 0x00, 0x00,
        (char)(sizeof(Mdata) >> 8), sizeof(Mdata) & 0xff,
        0x00, 0x00, 0x00 }, ATAPI_COMMAND::PIO_IN,
        &Mdata, sizeof(Mdata)
    };

  error = ATAPI_ExecCmd( &cmd_readmech );

  if (error) return error;

  pos = SwapLong(Mdata.MSF);
}

ERROR CD_Lock(bool lock)
{
  // Pause audio cd, Hold it, or start play from Holdposition (RESUME = 0x01)
  // p. 123
  uchar LOCK=0x00;
  if (lock == true)
      LOCK=0x01;

  ATAPI_COMMAND  cmd_pausecd =
    { { 0x1E, 0x00, 0x00,
        0x00, LOCK, 0x00,
        0x00, 0x00, 0x00,
        0x00, 0x00, 0x00}, ATAPI_COMMAND::NON_DATA };
  return  ATAPI_ExecCmd( &cmd_pausecd );
}

ERROR CD_Stop(void)
{
  // p. 195

  ATAPI_COMMAND  cmd_pausecd =
    { { 0x4E, 0x00, 0x00,
        0x00, 0x00, 0x00,
        0x00, 0x00, 0x00,
        0x00, 0x00, 0x00}, ATAPI_COMMAND::NON_DATA };
  return  ATAPI_ExecCmd( &cmd_pausecd );
}


ERROR CD_Eject(bool eject)
{
  // Pause audio cd, Hold it, or start play from Holdposition (RESUME = 0x01)
  // p. 123
  uchar EJECT=0x03;
  if (eject == true)
      EJECT=0x02;

  ATAPI_COMMAND  cmd_pausecd =
    { { 0x1B, 0x00, 0x00,
        0x00, EJECT, 0x00,
        0x00, 0x00, 0x00,
        0x00, 0x00, 0x00}, ATAPI_COMMAND::NON_DATA };
  return  ATAPI_ExecCmd( &cmd_pausecd );
}

ERROR CD_Sense(CD_SENSE &sense)
{
  // Get Infos from Device
  // p. 123
  ATAPI_COMMAND  cmd_sensecd =
    { { 0x03, 0x00, 0x00,
        0x00, 18,   0x00,
        0x00, 0x00, 0x00,
        0x00, 0x00, 0x00}, ATAPI_COMMAND::PIO_IN,
        &sense, sizeof(sense)};

  return  ATAPI_ExecCmd( &cmd_sensecd );
}
/************************/

ERROR  CD_StartUnit()
{
  // start stop unit, pp. 197
  ATAPI_COMMAND cmd_startunit =
    { { ATAPI_STARTSTOPUNIT,
        0x00, 0x00, 0x00,
        0x01, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00 },
      ATAPI_COMMAND::NON_DATA };

  return  ATAPI_ExecCmd( &cmd_startunit );
}


ERROR CD_TestUnitReady()
{
  // test unit ready, pp. 199
  ATAPI_COMMAND  cmd_testunit =
    { { ATAPI_TESTUNITREADY,
        0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00 },
        ATAPI_COMMAND::NON_DATA };

  return  ATAPI_ExecCmd( &cmd_testunit );
}


ERROR CD_ReadBlocks( void *buf, ulong lba, uint numblocks )
{
  // read(12), pp. 139
  ATAPI_COMMAND  cmd_read =
    { { ATAPI_READ12, 0x00,
        (char)(lba >> 24), (char)(lba >> 16),
        (char)(lba >>  8), (char)lba,
        0, 0, (char)(numblocks >>  8), (char)numblocks,
        0x00, 0x00 }, ATAPI_COMMAND::PIO_IN,
      buf, numblocks * CD_BLOCKSIZE };

  return  ATAPI_ExecCmd( &cmd_read );
}

ERROR CD_NoCRC( void )
{
  // play audio msf, pp. 128
  ATAPI_COMMAND  cmd_playaudiomsf =
    { { 0x01, 0x06, 0x31,
        0x00, 0x00, 0x00,
        0x00, 0x00, 0x00,
        0x00, 0x00, 0x00 }, ATAPI_COMMAND::NON_DATA };
  return  ATAPI_ExecCmd( &cmd_playaudiomsf );
}

ERROR  CD_Init()
{
  return ERR_OK;
}
