#ifndef CDROM_HPP
#define CDROM_HPP

#define  CD_1XSPEED                  176
#define  CD_BLOCKSIZE               2048
#define  CD_FRAMES_PER_SECOND         75L
#define  CD_FRAMES_PER_MINUTE       4500L

typedef struct  {
  uchar   Error_Code;
  uchar   Segement_Number;
  uchar   Sense_Key;
  uchar   Info[2];
  uchar   ASL;
  uchar   CInfo[2];
  uchar   ASC;
  uchar   ASCQ;
  uchar   Muell[10];
} CD_SENSE;


typedef struct  {
  uint    minute;
  uint    second;
  ulong    frame;
} CD_MSF;


typedef struct  {
  bool    data;
  ulong address;
  ulong length;
} CD_TRACK;


typedef struct  {
  uint        num_tracks;
  uint        num_audio_tracks;
  uint        num_data_tracks;
  ulong      volume_size;
  CD_TRACK   track[100];
} CD_TOC;


typedef struct  {
  uint     device_type;
  uchar    vendor_id  [ 8 + 1];
  uchar    product_id [16 + 1];
  uchar    product_rev[ 4 + 1];
} CD_DEVICE_ID;


extern  CD_MSF  LBA_To_MSF( unsigned long lba );
extern  unsigned long MSF_To_LBA( CD_MSF  msf );

extern  ERROR   CD_Inquiry( CD_DEVICE_ID *id  );
extern  ERROR   CD_ReadTOC( CD_TOC *toc );

extern  ERROR   CD_TestUnitReady();
extern  ERROR   CD_StartUnit();
extern  ERROR   CD_SetSpeed( int kb_per_sec );
extern  ERROR   CD_SetMaxSpeed( void ); //SF

extern  ERROR   CD_PlayAudioMSF( CD_MSF *start, CD_MSF *end );
extern  ERROR   CD_PlayAudioLBA( ulong start, ulong len);   //SF
extern  ERROR   CD_Scan( ulong start, uint dir);            //SF
extern  ERROR   CD_Pause(bool pause);                       //SF
extern  ERROR   CD_Position(ulong &pos);                    //SF 
extern  ERROR   CD_Lock(bool lock);									//SF
extern  ERROR   CD_Eject(bool eject);                       //SF
extern  ERROR   CD_Stop(void);										//SF			
extern  ERROR   CD_Sense(CD_SENSE &sense);                  //SF
extern  ERROR   CD_ReadBlocks( void *buf, unsigned long lba, unsigned int numblocks );
extern  ERROR   CD_NoCRC( void ); //SF
extern  ERROR   CD_Init();

#endif
