#ifndef ISO_STRUCTS_H
#define ISO_STRUCTS_H

// iso both byte order macros -----
#define ISOLONG(NAME)           \
    ulong   NAME;               \
    ulong   NAME ## _MOTOROLA

#define ISOSHORT(NAME)          \
    ushort  NAME;               \
    ushort  NAME ## _MOTOROLA


// iso volume descriptor types -----
#define  ISO_VDESC_BOOT      0
#define  ISO_VDESC_PVD       1
#define  ISO_VDESC_SVD       2
#define  ISO_VDESC_VPD       3
#define  ISO_VDESC_END     255


// iso standard id -----
#define  ISO_STANDARD_ID     "CD001"


// joliet character sets -----
#define  JOLIET_UCS2_1       "%/@"
#define  JOLIET_UCS2_2       "%/C"
#define  JOLIET_UCS2_3       "%/E"


// leave data fields unaligned -----
#pragma option -a1

typedef struct  {
  char        year1900, month, day;
  char        hour, minute, second;
  char        tz_offset;
} ISO_FILE_TIME;


typedef struct  {
  char        year[4];
  char        month[2];
  char        day[2];
  char        hour[2];
  char        minute[2];
  char        second[2];
  char        sec100[2];
  char        tz_offset;
} ISO_VDESC_TIME;


typedef struct  {
  uchar           recordLength;
  uchar           attribLength;
  ISOLONG         (extentLocation);
  ISOLONG         (dataLength);
  ISO_FILE_TIME   time;
  uchar           flags;
  uchar           unitSize;
  uchar           gapSize;
  ISOSHORT        (volumeSeqNumber);
  uchar           fileIDLength;
  char            fileID[1];
} ISO_FRECORD;


typedef struct  {
  uchar           type;
  char            stdID[5];
  char            version;
} ISO_VD_HEADER;


typedef struct  {
  ISO_VD_HEADER   header;
  char            reserved_8_2048[2041];
} ISO_VD_UNKNOWN;


typedef struct  {
  ISO_VD_HEADER   header;

  char            volumeFlags;           // svd only
  char            systemID[32];
  char            volumeID[32];

  char            reserved_73_80[8];

  ISOLONG         (volumeSpaceSize);
  char            escSequences[32];      // svd only
  ISOSHORT        (volumeSetSize);
  ISOSHORT        (volumeSeqNumber);
  ISOSHORT        (blockSize);

  ISOLONG         (pathTableSize);
  ISOSHORT        (lPathTable);
  ISOSHORT        (optLPathTable);
  ISOSHORT        (mPathTable);
  ISOSHORT        (optMPathTable);

  ISO_FRECORD     root;

  char            volumeSetID[128];
  char            publisherID[128];
  char            dataPreparerID[128];
  char            applicationID[128];

  char            copyrightFileID[37];
  char            abstractFileID[37];
  char            bibliographicFileID[37];

  ISO_VDESC_TIME  creationTime;
  ISO_VDESC_TIME  modificationTime;
  ISO_VDESC_TIME  expirationTime;
  ISO_VDESC_TIME  effectiveTime;

  char            fileStructVer;

  char            reserved_883;
  char            applicationUse[ 512 ];
  char            reserved_1396_2048[ 653 ];
}  ISO_PVD, ISO_SVD;

// restore data alignment -----
#pragma option -a.


#endif
