#ifndef CDPLAYER_HPP
#define CDPLAYER_HPP

#include <stdio.h>
#include "ide/mpplayer.hpp"
#include "ide/ata.hpp"
#include "ide/cdrom.hpp"
#include "ide/iso9660.hpp"
#include "ks0108.h"
#include "mas/masi2c.hpp"

#define EJECTED               0x00  //Ob das geht?!
#define STOP     					0x01
#define PLAYING  					0x02
#define PAUSEING 					0x03
#define FAST_SCAN             0x04

#define PLAY   0x3b5
#define PAUSE  0x3b0
#define NEXT   0x3a0
#define PREV   0x3a1

#define NO_ERR						0
#define TRACK_TO_LOW				1
#define TRACK_TO_HIGH			2
#define POS_TO_LOW				1
#define POS_TO_HIGH           2

#define YA_STOP    0x551
#define YA_PLAY    0x55C
#define YA_PAUSE   0x55D
#define YA_BACK    0x51B
#define YA_NEXT    0x518
#define YA_VOLU    0x544
#define YA_VOLD    0x51D
#define YA_BALL    0x51C
#define YA_BALR    0x548

//Funktionstasten States
#define VOL     0
#define BASS    1
#define TREB    2
#define BAL     3

class  CD_Player {
 uint    FState;			 //Gibt den Status der Funktionstage an!

 uint    Last_Track;     //Nach dem Init, Maximale Anzahl der Tracks...
 ulong	Last_Pos;		 //Letzter LBA der CD Wiedergabe...
 CD_MSF  Last_MSF;
 uint    Play_Track;     //Aktueller Track der gespielt wird.
 ulong   Play_Pos;       //Aktueller LBA der Wiedergabe
 CD_MSF  Play_MSF;
 uchar   State;          //Gibt an, ob eine Wiedergabe stattfindet...
 CD_TOC  toc;            //Enthlt TOC der CD-ROM...
 CD_MSF  TimeToPlay, TimeLen, TimePos;  //Die verschiedenen Zeiten...
 char    infotext[80];   //Zum erstellen des Infotextes frs LCD...
 uint    len;
 uint    slen;

 public:

 void FMode(void) {
  FState++;
  if (FState==BAL+1) FState=0;
 }

 void SText(char *kette)
 {
  for (int i=0;i<16;i++)
    {
     if ((i+len) >= strlen(kette))
        TLCD_send(' ',true);
     else
        TLCD_send(kette[i+len],true);
    }
  len++;
  if (len>=strlen(kette)) len = 0;
  RTX_Sleep_Time(100);
 }


 void Stop(void)
  {
     switch (State) {
       case PLAYING:
       case FAST_SCAN:
       case PAUSEING: CD_Stop(); State = STOP; break;
       case STOP: CD_Lock(false); CD_Eject(true); State = EJECTED; break;
       case EJECTED: CD_Eject(false); State= STOP;
                     //Warten bis CD wieder eingebucht ist!
                     while (!kbhit()) {
                       if (CD_TestUnitReady() == ERR_OK)
                           break;
                       RTX_Sleep_Time(100);
                     }

                     CD_Lock(true);
                     CD_PLInit();
                     break;
     }
  }

 void Pause(void) {
  switch (State) {
  case PLAYING:  CD_Pause(true); State = PAUSEING; break;
  case PAUSEING: CD_Pause(false); State = PLAYING; break;
  }
 }

 void Next(void) {
  switch (State) {
   case PLAYING:  if (TRACK_TO_HIGH == CD_Play_Track(++Play_Track)) CD_Play_Track(1); break;
   case PAUSEING: if (TRACK_TO_HIGH == CD_Play_Track(++Play_Track)) {CD_Play_Track(1);} Pause(); break;
  }
 }

 void Back(void) {
  switch (State) {
   case PLAYING:  if (TRACK_TO_LOW == CD_Play_Track(--Play_Track)) CD_Play_Track(Last_Track); break;
   case PAUSEING: if (TRACK_TO_LOW == CD_Play_Track(--Play_Track)) {CD_Play_Track(Last_Track);} Pause(); break;
  }
 }

 uchar FillPosInfo(void) {
   CD_Position(Play_Pos);
   //Auch > 0 da der letzte Track einen Nachfolger auf 0 hat!
   //if (toc.track[track].address < pos) && (toc.track[track].address > 0) track++;
    if ((toc.track[Play_Track-1].address + toc.track[Play_Track-1].length) < Play_Pos) Play_Track++;
    if (toc.track[Play_Track-1].address > Play_Pos) Play_Track--;
    if ((Play_Track >= Last_Track) || Play_Track <= 0);
         {
           //CD ist am Ende angekommen!
           //Was nun Herr Programmierer!
           //State = STOP;
         }

   TimePos=LBA_To_MSF(Play_Pos - toc.track[Play_Track-1].address);
   TimeLen=LBA_To_MSF(toc.track[Play_Track-1].length);
   TimeToPlay=LBA_To_MSF(toc.track[Play_Track-1].length - (Play_Pos - toc.track[Play_Track-1].address));

   return NO_ERR;
 }

 //Spielt die CD anhand einer bergebenen Tracknummer ab.
 uchar CD_Play_Track(uint Track) {
  printf("Play Track: %d\n", Track);

  if (Track <= 0)
     return TRACK_TO_LOW;
  if (Track > Last_Track)
     return TRACK_TO_HIGH;

  printf("-> Play Track: %d\n", Track);

  Play_Track = Track;
  State=PLAYING;
  return CD_Play_LBA(toc.track[Track-1].address);
 }

 //Spielt die CD anhand einer bergebenen LBA Adresse ab.
 uchar CD_Play_LBA(ulong spos) {
   printf("Will Play LBA: %d\n", spos);

   /*Der Trick der aktuellen Kopierschutzfunktionen ist, dass angeblich ab Pos
     -1 gespielt werden soll, aber nach Spec muss eine Audio CD bei 0:0.2 anfangen!!!*/
   if (spos == -1)
    {
     spos = 0;  //Fr so ne blde LBA, gibts einen Drauf *SF
/*   Play_Track = 1; //Jetzt sehen wir mal was eigendlich passiert!!!
     for (int z=0;z<=Last_Track;z++)
      {
        toc.track[z].length-=2;
      }*/
     toc.track[0].address = 0; //NEU ?!?!
    }

   if (spos < 0)  //Fixed 01.02.2002 SF
     return POS_TO_LOW;
   if ((spos != 0xFFFFFFFFL) && (spos > Last_Pos))
     return POS_TO_HIGH;
   printf("--> Play LBA: %d\n", spos);

   CD_Lock(true);
   switch (State) {
   case FAST_SCAN:
      CD_PlayAudioLBA(0xFFFFFFFFL, Last_Pos);
      break;
   case PAUSEING:
      Pause();
      break;
   default:
//    CD_PlayAudioLBA(spos, Last_Pos);
      Play_MSF = LBA_To_MSF(spos);
//    Last_MSF.minute = 30;
//    Last_MSF.second  = 30;
      printf("Last_MSF: %u %u %u\n", Last_MSF.minute, Last_MSF.second, Last_MSF.frame);
      CD_PlayAudioMSF(&Play_MSF, &Last_MSF);
      break;
   }
   State = PLAYING;
   return NO_ERR;
 }

 void LCD_INFO(void) {
      FillPosInfo();

      switch (State) {
      case PAUSEING:  sprintf(infotext, " --- PAUSE --- "); break;
      case STOP:      sprintf(infotext, " ---- STOP ---- "); break;
      case EJECTED:   sprintf(infotext, "Insert CD-ROM!"); break;
      case PLAYING:
                      switch (FState) {
                       case VOL:   sprintf(infotext,"%02d:%02d %02d:%02d V:%02X", TimePos.minute, TimePos.second,TimeLen.minute, TimeLen.second, getVol() /* TimeToPlay.minute, TimeToPlay.second */); break;
                       case BASS:  sprintf(infotext,"%02d:%02d %02d:%02d B:%02X", TimePos.minute, TimePos.second,TimeLen.minute, TimeLen.second, getBass() /* TimeToPlay.minute, TimeToPlay.second */); break;
                       case TREB:  sprintf(infotext,"%02d:%02d %02d:%02d T:%02X", TimePos.minute, TimePos.second,TimeLen.minute, TimeLen.second, getTreb() /* TimeToPlay.minute, TimeToPlay.second */); break;
                       case BAL:   sprintf(infotext,"%02d:%02d %02d:%02d BA%02X", TimePos.minute, TimePos.second,TimeLen.minute, TimeLen.second, getBal() /* TimeToPlay.minute, TimeToPlay.second */); break;
                       }
                      break;
      case FAST_SCAN: sprintf(infotext,"%02d:%02d -- SCAN --", TimePos.minute, TimePos.second); break;
      }
      pos_cursor(0, 0);
      TLCD_Kette(infotext);

      switch (State) {
      case STOP:    sprintf(infotext, "Press STOP to Eject CD-ROM now, or press PLAY to Play CD-ROM now.\0x0"); break;
      case EJECTED: sprintf(infotext, "Press STOP to Load CD-ROM now."); break;
      case FAST_SCAN:
      case PAUSEING:
      case PLAYING:
           sprintf(infotext,"Track: %02d von %02u ", Play_Track, Last_Track); break;
      }
      pos_cursor(0, 1);
      if ((State==STOP) || (State==EJECTED))
           SText(infotext);
      else
      TLCD_Kette(infotext);
 }

 void RC_KILL() {
   while (RC_KeyAvail())
      unsigned long key = RC_GetKey();
 }

 void inter(void) {
  int bt;
  int ToDo;
   while (!kbhit())
   {
      while (RC_KeyAvail()) {
      unsigned long key = RC_GetKey();
//       printf("unknown RC code: %lx\n", key);
      switch (key /*& (RCKEY_PROTO | RCKEY_CODE)*/) {
        case (RCKEY_YA | 0x39e):   // SCAN BACK
            CD_Scan(Play_Pos,0); RC_KILL(); State = FAST_SCAN;
            break;
        case (RCKEY_YA | 0x39f):   // SCAN BACK
            CD_Scan(Play_Pos,1); RC_KILL(); State = FAST_SCAN;
            break;
        case (RCKEY_YA | /*0x3a1*/ YA_BACK):   // BACK
           TLCD_clear(); TLCD_Kette("Back"); Back(); RC_KILL();
          break;
        case (RCKEY_YA | /*0x3a0*/ YA_NEXT):   // NEXT
           Next(); RC_KILL();
          break;
        case (RCKEY_YA | /*0x3b0*/ YA_PAUSE):   // PAUSE
           Pause(); RC_KILL();
          break;
        case (RCKEY_YA | /*0x3b5*/ YA_PLAY):   // Play!
           CD_Play_LBA(Play_Pos); RC_KILL();
           break;
        case (RCKEY_YA | /*0x3b5*/ YA_VOLU):   // VOL UP!
           FState = VOL; mas_incVol(); RC_KILL();
           break;
        case (RCKEY_YA | /*0x3b5*/ YA_VOLD):   // VOL DOWN!
           FState = VOL;  mas_decVol(); RC_KILL();
           break;
        case (RCKEY_YA | /*0x3b5*/ YA_BALL):   // VOL UP!
           FState = BAL; mas_decBal(); RC_KILL();
           break;
        case (RCKEY_YA | /*0x3b5*/ YA_BALR):   // VOL DOWN!
           FState = BAL; mas_incBal(); RC_KILL();
           break;

        default:
          printf("unknown RC code: %lx\n", key);
          break;
      }
    }

      /********************** PAPA TASTEN ***********************************/
   	bt = inp(0x600);
      if ((bt & 0x01)==0) { TLCD_clear(); TLCD_Kette("Back"); Back(); while (inp(0x600)!=0xFF); }
      if ((bt & 0x02)==0) { TLCD_clear(); TLCD_Kette("Next"); Next(); while (inp(0x600)!=0xFF); }
      if ((bt & 0x04)==0) { TLCD_clear(); TLCD_Kette("Stop"); Stop(); while (inp(0x600)!=0xFF); }
      if ((bt & 0x08)==0) { TLCD_clear(); TLCD_Kette("Play"); CD_Play_LBA(Play_Pos); while (inp(0x600)!=0xFF); }
      if ((bt & 0x10)==0) { TLCD_clear(); TLCD_Kette("Pause"); Pause(); while (inp(0x600)!=0xFF);}
//      if ((bt & 0x20)==0) { TLCD_clear(); TLCD_Kette("Ende"); while (inp(0x600)!=0xFF); return;}
      if ((bt & 0x20)==0) { FMode(); while (inp(0x600)!=0xFF);}
      if ((bt & 0x40)==0) {
                            switch (FState) {
                            case VOL  :   mas_decVol(); break;
                            case BASS :   mas_decBass(); break;
                            case TREB :   mas_decTreb(); break;
                            case BAL  :	mas_decBal(); break;
                            }
      						  }
      if ((bt & 0x80)==0) {
                            switch (FState) {
                            case VOL  :   mas_incVol(); break;
                            case BASS :   mas_incBass(); break;
                            case TREB :   mas_incTreb(); break;
									 case BAL  :	mas_incBal(); break;
                            }
      						  }
      LCD_INFO();                }
 }

 CD_Player(bool start_play=false) {
   CD_PLInit(start_play);
 }

 CD_PLInit(bool start_play=false) {
   FState = 0;
   //TOC einlesen...
   CD_ReadTOC( &toc );
   Last_Track = toc.num_tracks;
	Last_Pos   = toc.volume_size;
   Last_MSF   = LBA_To_MSF(toc.volume_size);

   Play_Track = 1;     /*Fr Kopierschutz CDs!!! */
   Play_Pos   = 0;
   len        = 0;
   slen       = 0;

   State = PLAYING;
//   CD_Play_LBA(2); //Fixed ? I don't know it? Why does this do?
   CD_Play_Track(1); //Nach dem Laden gehts Los!!!
   printf("Init CDPlayer...\n");
   printf("Last_Track: %d\n", Last_Track);
   printf("Last_Pos: %d\n", Last_Pos);
   printf("Last_MSF: %u %u %u\n", Last_MSF.minute, Last_MSF.second, Last_MSF.frame);
 }

};

#endif CDPLAYER_HPP
