/****************************************************************************
*
* (C) 2000 by BECK IPC GmbH
*
*  BECK IPC GmbH
*  Garbenheimerstr. 38
*  D-35578 Wetzlar
*
*  Phone : (49)-6441-905-240
*  Fax   : (49)-6441-905-245
*
* ---------------------------------------------------------------------------
* Module      : IPC@CHIP SC12 API :rtos.h
*
* Function    :   prototypes and constants RTOS API
*                 Use model large at your program, if you use this include file.
*                 See html documentation of the RTOS API for details.
* Author        : bartat
* Date          : 19.01.00
* ---------------------------------------------------------------------------

$Header: RTOS.H, 14, 14.02.2002 10:47:27, Christoph Stoidner$

$Log:
 14   IPC@CHIP  1.13        14.02.2002 10:47:27  Christoph Stoidner add Find
      Semaphore Function
 13   IPC@CHIP  1.12        16.01.2002 18:40:50  Markus Bartat   add a semicolon
      after prototype declarartion at
      RTX_Disable_Task_Scheduling(void)
      and
       RTX_Enable_Task_Scheduling(void);
 12   IPC@CHIP  1.11        16.01.2002 11:45:06  Christoph Stoidner add starteam
      directives
 11   IPC@CHIP  1.10        14.01.2002 13:18:38  Christoph Stoidner add new
      functions
 10   IPC@CHIP  1.9         23.07.2001 10:17:37  Christoph Stoidner comment
 9    IPC@CHIP  1.8         02.07.2001 13:29:07  Markus Bartat   delete the send
      message/taskstart functions
 8    IPC@CHIP  1.7         19.06.2001 13:40:11  Markus Bartat   edit
 7    IPC@CHIP  1.6         19.06.2001 13:35:08  Markus Bartat   start: Added
      comments to the c functions
 6    IPC@CHIP  1.5         01.06.2001 12:20:27  Markus Bartat   added new
      functions
 5    IPC@CHIP  1.4         31.05.2001 16:18:26  Christoph Stoidner
 4    IPC@CHIP  1.3         31.05.2001 14:27:45  Christoph Stoidner
 3    IPC@CHIP  1.2         29.05.2001 16:03:58  Markus Bartat   added API call
      0x11 Create a task without starting it
 2    IPC@CHIP  1.1         15.02.2001 15:16:52  Christoph Stoidner
 1    IPC@CHIP  1.0         14.02.2001 16:09:47  Christoph Stoidner
$


* History       :
*
*  Vx.yy                   Author  Changes
*
*             19.01.00       mb    Create
*             22.07.00       mb    add timer procedure calls
*             20.08.00       mb    add task monitoring API calls
*             24.08.00       mb    add API calls suspend and resume tasks
*             23.10.00       mb    add event group calls
*             26.10.00       mb    add message exchange calls
*             29.05.01       mb    add task create call without running the task
*             01.06.01       mb    add get task state call without using taskmonitoring
*****************************************************************************/

#ifndef _RTOS_H___
#define _RTOS_H___

/*************************************************************************/
#include <DOS.H>
#include "rtxapi.h"
/*************************************************************************/
//defines
/*************************************************************************/
#define RTOSVECT     0xAD

#ifdef _MSC_VER /* If Microsoft C Compiler is used, define new FP_SEG and FP_OFF Macro */
	#ifdef FP_SEG
  		#undef FP_SEG
	#endif
	#define FP_SEG(ptr)    ((unsigned int)(((unsigned long)ptr)>>16))

	#ifdef FP_OFF
  		#undef FP_OFF
	#endif
	#define FP_OFF(ptr)    ((unsigned int)(((unsigned long)ptr)&0xFFFF))
#endif


/*************************************************************************/
//prototypes
/*************************************************************************/
int RTX_Create_Task(int  far * taskID, TaskDefBlock  far  * taskdefblock);
int RTX_Create_Task_Without_Run(int far * taskID, TaskDefBlock  far * taskdefblock);
/*************************************************************************/
//Delete a task from the system
/*************************************************************************/
int RTX_Delete_Task(int taskID);
/*************************************************************************/
//Stop and kill a task
/*************************************************************************/
int RTX_Kill_Task(int taskID);
/*************************************************************************/
//Go to sleep for a defined time
/*************************************************************************/
int RTX_Sleep_Time(unsigned int time_ms);
/*************************************************************************/
//Go to sleep, until  wakerequest
/*************************************************************************/
int RTX_Sleep_Request(void);
/*************************************************************************/
//Wakeup a sleeping task
/*************************************************************************/
int RTX_Wakeup(int taskID);
/*************************************************************************/
//End execution of task by itself, this functions never returns
/*************************************************************************/
int RTX_End_Execution(void);
/*************************************************************************/
//Suspend/Resume task execution
/*************************************************************************/
int RTX_Suspend_Task(int taskID);
int RTX_Resume_Task(int taskID);
/*************************************************************************/
//Change taskprio
/*************************************************************************/
int RTX_Change_TaskPrio(int taskID, int prio, int  far  *error);
/*************************************************************************/
//Get the ID of the current running task
/*************************************************************************/
int RTX_Get_TaskID(void);
/*************************************************************************/
//file access
/*************************************************************************/
int RTX_Access_Filesystem(void);
/*************************************************************************/
//Get state of a task, using the task monitoring
/*************************************************************************/
int RTX_Get_Task_State(char * taskname, Task_StateData * taskdata, int * error);
/*************************************************************************/
//Get state of a task, without using the task monitoring
/*************************************************************************/
int RTX_Get_Task_State_Ext(char * taskname, unsigned int * taskstate, int * error);
/*************************************************************************/
//Get task  list
/*************************************************************************/
int RTX_Get_Task_List(TaskList * tasklist, int length);
/*************************************************************************/
//Start taskmonitoring
/*************************************************************************/
void RTX_Start_Task_Monitor(void);
/*************************************************************************/
//Start taskmonitoring
/*************************************************************************/
void RTX_Stop_Task_Monitor(void);
/*************************************************************************/
//Restart a  task, killed with call 0x02
/*************************************************************************/
int RTX_Restart_Task(int taskID);
/*************************************************************************/
//Disable the Task Scheduling
/*************************************************************************/
void RTX_Disable_Task_Scheduling(void);
/*************************************************************************/
//Enable the Task Scheduling
/*************************************************************************/
void RTX_Enable_Task_Scheduling(void);
/*************************************************************************/
//Wake up a task, which sends direct a message
/*************************************************************************/
int RTX_Wake_Msg_Receipt(void);
/*************************************************************************/
//Create a semaphore
/*************************************************************************/
int RTX_Create_Sem(int  far  * semID, char  far  * name, int initvalue);
/*************************************************************************/
//Delete a  semaphore
/*************************************************************************/
int RTX_Delete_Sem(int semID);
/*************************************************************************/
//Free a  resource semaphore
/*************************************************************************/
int RTX_Free_Sem(int semID);
/*************************************************************************/
//Get a  counting semaphore
/*************************************************************************/
int RTX_Get_Sem(int semID);
/*************************************************************************/
//Release a  resoure semaphore
/*************************************************************************/
int RTX_Release_Sem(int semID);
/*************************************************************************/
//Reserve a  resoure semaphore (optional timeout)
/*************************************************************************/
int RTX_Reserve_Sem(int semID, long   far  * time_ms);
/*************************************************************************/
//Signal a  counting semaphore
/*************************************************************************/
int RTX_Signal_Sem(int semID);
/*************************************************************************/
//Wait on a  counting semaphore (optional timeout)
/*************************************************************************/
int RTX_Wait_Sem(int semID, long  far  *time_ms);
/*************************************************************************/
//Find an semaphore, specified by 4 char name tag
/*************************************************************************/
int RTX_Find_Sem(int * ID, char * name);
/*************************************************************************/
//Get system time and date
/*************************************************************************/
int RTX_Get_TimeDate(TimeDate_Structure  far  * td);
/*************************************************************************/
//Set system time and date
/*************************************************************************/
int RTX_Set_TimeDate(TimeDate_Structure  far  * td);
/*************************************************************************/
//Set system ticks
/*************************************************************************/
void RTX_Get_System_Ticks(unsigned long far * ticks);
/*************************************************************************/
//Install timer procedure
/*************************************************************************/
int RTX_Install_Timer(TimerProc_Structure far * TProcPtr);
/*************************************************************************/
//Remove timer procedure
/*************************************************************************/
int RTX_Remove_Timer(unsigned int timerID);
/*************************************************************************/
//Start periodic execution of timer procedure
/*************************************************************************/
int RTX_Start_Timer(unsigned int timerID);
/*************************************************************************/
//Stop periodic execution of timer procedure
/*************************************************************************/
int RTX_Stop_Timer(unsigned int timerID);

/*************************************************************************/
//Create event group
/*************************************************************************/
int RTX_Create_EventGroup(unsigned int * ID, char * name, int init_value);
/*************************************************************************/
//Delete an event group
/*************************************************************************/
int RTX_Delete_EventGroup(unsigned int ID);
/*************************************************************************/
//Signal events group
/*************************************************************************/
int RTX_Signal_Events(unsigned int ID, unsigned int event_mask, unsigned event_value);
/*************************************************************************/
//Wait for events
/*************************************************************************/
int RTX_Wait_For_Event(unsigned int ID, RTX_Wait_Event * event_ptr);
/*************************************************************************/
//Read current events on a group
/*************************************************************************/
int RTX_Get_EventGroup_State(unsigned int ID, unsigned int * Event);
/*************************************************************************/
//Get the saved event flags/bits
/*************************************************************************/
int RTX_Get_Saved_Events(unsigned int * Event);
/*************************************************************************/
//Find an event group, specified by 4 char name tag
/*************************************************************************/
int RTX_Find_EventGroup(unsigned int * ID, char * name);
/*************************************************************************/
//Create message exchange
/*************************************************************************/
int RTX_Create_Msg(RTX_Msg * msg_ptr);
/*************************************************************************/
//Delete a message exchange
/*************************************************************************/
int RTX_Delete_Msg(unsigned int ID);
/*************************************************************************/
//Send a message to a message exchange
/*************************************************************************/
int RTX_Send_Msg(unsigned int ID, int prio, char * msg);
/*************************************************************************/
//Get a message from a message exchange, no wait
/*************************************************************************/
int RTX_Get_Msg(unsigned int ID, char * msg);
/*************************************************************************/
//Wait for a message, optional timeout
/*************************************************************************/
int RTX_Wait_For_Msg(RTX_Wait_Msg * msg_ptr);
/*************************************************************************/
//Find an message exchange, specified by 4 char name tag
/*************************************************************************/
int RTX_Find_Msg(unsigned int * ID, char * name);
/*************************************************************************/
#endif /* _RTOS_H___ */
/*************************************************************************/
//end rtos.h
